import java.io.*;
import java.util.*;
import java.lang.Boolean;
import java.text.*;					// Used for Date Formatting
import java.util.Random; 
//import java.util.ResourceBundle;

/*
	Document is an abstract class that serves as ancestor for basic file IO functions. Shared file functionality can
	be found in this class. Many of the methods found here are also included in tools.java.
*/

public abstract class Document
{	
//    protected ResourceBundle resbundle;
    
	public String RN = new String("\r\n");
	public String BLANK = new String("");
	public String TAB = new String("\t");
	public String COLON = new String(":");
	
	public String NOTE = new String("NOTE");
	public String ACCIDENTAL = new String("ACCIDENTAL");    
	public String REST = new String("REST");	
	public String KEY = new String("KEY");
	public String FIFTHS = new String("FIFTHS");
	public String CLEF = new String("CLEF");

	public String COMMENTSTART = new String(";");
	public String GAMUTSQUARESTART = new String("START");
	public String NUMBER = new String("NUMBER");
	public String FLIPSETNUMBER = new String("FLIPSETNUMBER");
	public String METER = new String("METER");
	public String NOTES = new String("NOTES");
	public String DURATIONS = new String("DURATIONS");

	public String KEYSIGNATURE = new String("KEYSIGNATURE");
	public String END = new String("END");	
	public String WEIGHTING = new String("WEIGHTING");
	
	public String XML = new String("XML");	
	public String SCORE = new String("SCORE");			
	
    public String OUTPUTFILENAME = new String("OUTPUTFILENAME");
    public String OUTPUTFILEPATH = new String("OUTPUTFILEPATH");
	public String OUTPUTTYPE = new String("OUTPUTTYPE");
	
//	public String TR = new String("TR");
//	public String BA = new String("BA");		
//	public String AL = new String("AL");			
//	public String TE = new String("TE");

	//public String TREBLE = new String("T");
	public String TREBLE = new String("TREBLE");
	public String TREBLESIGN = new String("G");
	public String TREBLELINE = new String("2");	
	//public String BASS = new String("B");	
	public String BASS = new String("BASS");
	public String BASSSIGN = new String("F");
	public String BASSLINE = new String("4");
	//public String ALTO = new String("A");
	public String ALTO = new String("ALTO");
	public String ALTOSIGN = new String("C");
	public String ALTOLINE = new String("3");				
	//public String TENOR = new String("N");
	public String TENOR = new String("TENOR");
	public String TENORSIGN = new String("G");
	public String TENORLINE = new String("2");	
		// also <clef-octave-change>-1</clef-octave-change>
	public String SLASH = new String("/");
	public String NOTENAMES = new String("ABCDEFGR");
	public String ACCIDENTALNAMES = new String("Nb#x");
	
	public String SHARP = new String("#");
	public String DOUBLESHARP = new String("x");
    public String FLAT = new String("b");
    public String DOUBLEFLAT = new String("bb");
	public String NATURAL = new String("N");
	
	public String SPACE = new String(" ");
	public String SYSTEMS = new String("SYSTEMS");
	
	public String COMPOSER = new String("COMPOSER");
	public String PIECENAME = new String("PIECENAME");	
	public String POET = new String("POET");
	public String RIGHTS = new String("RIGHTS");	
	public String SOFTWARE = new String("SOFTWARE");	
	public String ENCODINGDATE = new String("ENCODINGDATE");
	public String SIZE = new String("SIZE");	
	public String SYSTEMCLEFS = new String("SYSTEMCLEFS");
	public String PARTS = new String("PARTS");
	public String NULL = new String("null");
	public String PROCESS = new String("PROCESS");
	public String GROUP = new String("GROUP");
	public String MARKUP = new String("MARKUP");
	
	public String STEP = new String("step");
	public String OCTAVE = new String("octave");
	public String ALTER = new String("alter");
	public String DURATION = new String("duration");
	public String TYPE = new String("type");
	public String ACCIDENTALTYPE = new String("accidental");	
	public String RESTVALUE = new String("R");
	
	public String DYNAMICS = new String("DYNAMICS");
	public String PPPP = new String("pppp");
	public String PPP = new String("ppp");
	public String PP = new String("pp");
	public String P = new String("p");
	public String MP = new String("mp");
	public String MF = new String("mf");
	public String F = new String("f");
	public String FF = new String("ff");
	public String FFF = new String("fff");
	public String FFFF = new String("ffff");
    public String SFPP = new String("sfpp");
    public String SFFP = new String("sffp");
    public String SFP = new String("sfp");
    public String SF = new String("sf");
    public String FP = new String("fp");
    public String FZ = new String("fz");
    public String SFZ = new String("sfz");
    public String SFFZ = new String("sffz");
    public String EMPTY = new String("empty");
    
    public String DOT = new String(".");
    
    public String STACCATO = new String("staccato");
    public String TENUTO = new String("tenuto");
    public String ACCENT = new String("accent");
    
	public String ARTICULATION_STACCATO=new String(".");
	public String ARTICULATION_TENUTO= new String("_");
	public String ARTICULATION_ACCENT = new String(">");
	
	public String LILYPOND = new String("LILYPOND");
	public String MUSICXML = new String("MUSICXML");
	
    public String inputFilenameText = new String();
    
	boolean bDebug = false;
	
    public Integer pitch_C = new Integer(0);
    public Integer pitch_D = new Integer(2);
    public Integer pitch_E = new Integer(4);
    public Integer pitch_F = new Integer(5);
    public Integer pitch_G = new Integer(6);
    public Integer pitch_A = new Integer(8);
    public Integer pitch_B = new Integer(10);
	public Integer accidental_sharp = new Integer(1);
	public Integer accidental_flat = new Integer(-1);
	public Integer octave = new Integer(12);
	
	public Hashtable pitchHashtable = new Hashtable();
	
	public void buildPitchHashtable()
	{
		pitchHashtable.put("C", pitch_C);
		pitchHashtable.put("D", pitch_D);
		pitchHashtable.put("E", pitch_E);
		pitchHashtable.put("F", pitch_F);
		pitchHashtable.put("G", pitch_G);
		pitchHashtable.put("A", pitch_A);
		pitchHashtable.put("B", pitch_B);		
	}
	
	// Pass in a single parameter and filename and return the data associated with that param
	public String GetParameterData(String param, String filename)
	{		
		String result = new String();
		
		// get arraylist of all lines found in file
		List Lines = new ArrayList(getLines(filename));
		
		// Create master List that holds Lists of all valid line values in each node (valid = has data in them)
		List validLineValues = new ArrayList();
		
		// for each line in Lines, get arraylist of values and place them in arraylist
		int linesSize = Lines.size();
		
		String currentValue = new String();
		
		// Loop through each line, get arraylist of values, then add that arraylist to arraylist of all values
		for (int i=0; i<linesSize; i++)
		{
			// If current line isValid (i.e. Doesn't start with space or comment)...
			if(isValid(Lines.get(i).toString()))
			{				
				// Pass string from Line in and get back an arraylist of all individual values
				List currentLineValues = getLineValues(Lines.get(i).toString());
			
				currentValue = (String)currentLineValues.get(0);
				
				// compare first value to param
				if(param.compareToIgnoreCase(currentValue)==0)
				{
					result=currentValue;
					i=linesSize;
				}				
			}
		}
		
		return result;
	}
	
	/**
	*	Public method that takes a filename (filename includes filepath) and returns an ArrayList
	*	of all values on all lines of the file. Essentially, it builds an arraylist of arraylists
	*	where the primary list contains a secondary list of values. Each value is a word separated
	* 	by a space and conforming to the validations in the isValid method.
	*
	* 	@param	filename	String filename which also contains the full file path
	*	@return	List		ArrayList of filevalues
	**/
	public List GetFileValues(String filename)
	{
		
		// get arraylist of all lines found in file
		List Lines = new ArrayList(getLines(filename));
		
		// Create master List that holds Lists of all valid line values in each node (valid = has data in them)
		List validLineValues = new ArrayList();
		
		// for each line in Lines, get arraylist of values and place them in arraylist
		// Size of 
		int linesSize = Lines.size();
		
		// Loop through each line, get arraylist of values, then add that arraylist to arraylist of all values
		for (int i=0; i<linesSize; i++)
		{									
			// If current line isValid (i.e. Doesn't start with space or comment)...
			if(isValid(Lines.get(i).toString()))
			{
				// Pass string from Line in and get back an arraylist of all individual values
				List currentLineValues = getLineValues(Lines.get(i).toString());
			
				// Add arraylist of individual values to master arraylist
				validLineValues.add(currentLineValues);
			}
		}
		
		return validLineValues;
	}

	/**
	*	private method that sends the values of the incoming List to the System.out
	*
	*	@param	values	List of String values
	**/
	private void writeArrayList(List values)
	{
		for(int i=0; i<values.size(); i++)
		{
			//printSystemOut("Document.java : writeArrayList : " + values.get(i).toString());
		}
	}
	

	/**
	* 	Private method that gets values from a String (separated by spaces) and returns them
	*	in a List object (should make this return an ArrayList).
	*
	*	@param	line	String value to be converted to an ArrayList.
	*	@return	List	Object containing individual values from a String.
	**/
	private List getLineValues(String line)
	{
		// Array list that will hold all values for this line
		List values = new ArrayList();
		
		// temporary result stringbuffer
		StringBuffer result = new StringBuffer();
		
		int lineLength = line.length();
		
		// Step through each char, writing to buffer until ' ' to get first word
		for (int i=0; i< lineLength;i++)
		{
			char currentChar = line.charAt(i);
			
			// Add character to buffer
			if (currentChar != ' ') 
			{
				result.append(currentChar);

				// Check to see if we're at end of line
				if(i==(lineLength-1))
				{
					values.add(result.toString());
				}
			}
			else// when we hit the first ' ' , add values, delete written chars from buffer and step out
			{
				// Ignore leading and multiple spaces
				if(result.length() != 0)
				{
					// Add value to ArrayList
					values.add(result.toString());
				
					// Clear string buffer for other values in this line
					result.delete(0, result.length());
				}
			}
							
		}	// end of "for(int j=0..."
				
		return values;
	}
	
	public String ReadFile(String filename)
	{
		StringBuffer returnValue = new StringBuffer();
		
		String thisLine;
		List curLines;
		
		curLines = new ArrayList();
		
		//inputFilenameText = "";
		
		try 
		{
		    BufferedReader br = new BufferedReader(new FileReader(filename));
			   
		    //printSystemOut("printSystemOut(br.toString()); = "+br.toString());
		    
			boolean done = false;
		
			while (!done)
			{				
				if ((thisLine = br.readLine()) == null)
				{					
		  		  	//printSystemOut("read past end of file");
		    			done = true;
		    		//return;
				}
				else
				{					
					returnValue.append(thisLine+nextline());	
				}
			}						
		}
		catch (IOException e) 
		{
	    	System.err.println("Error: "+e);
		} //end try
				
		
		return returnValue.toString();		
	}
	
	/**
	*	Private method that adds values to a List object from a String. Currently used  
	*	only for test purposes.
	*
	*	@param	filename	File name and full path to be checked.
	*	@return	List		List object containing string's values.
	**/		
	private List getLines(String filename)
    {
		String thisLine;
		List curLines;
		
		curLines = new ArrayList();
		
		// only necessary for writing input file to String
		ArrayList inputText = new ArrayList();
		
		//inputFilenameText = "";
		
		try 
		{
		    BufferedReader br = new BufferedReader(new FileReader(filename));
			   
		    //printSystemOut("printSystemOut(br.toString()); = "+br.toString());
		    
			boolean done = false;
		
			while (!done)
			{				
				if ((thisLine = br.readLine()) == null)
				{					
		  		  	//printSystemOut("read past end of file");
		    			done = true;
		    		//return;
				}
				else
				{					
					inputText.add(thisLine+nextline());					
					curLines.add(thisLine);					
				}
			}
			
			//printSystemOut(inputFilenameText);
		
		}
		catch (IOException e) 
		{
	    	System.err.println("Error: "+e);
		} //end try
		
		this.inputFilenameText = inputText.toString();
		
		return curLines;
		
	}    
	
	/**
	*	Private method that checks the validity of a String value based on arbitrary rules. 
	*	In this case the rules are directly applicable to the CSound programming language and
	*	valid file formats.
	*	
	*	@param	line	String value to be validated.
	*	@return	boolean	True/false as to whether or not the value is valid	
	**/	
	private boolean isValid(String line)
	{
		boolean result = false;
		int lineLength = line.length();
		
		// If there are chars in the line...
		if(lineLength > 0)
		{
			char firstChar = line.charAt(0);
                        
//6.03.02 RKH - now return comments, filter them out later on so we can get header values                
//			// Skip to next line if char(0) is blank or comment
//			if(firstChar != ';')
//			{
                        result = true;
//			}
		}
		return result;
	}


	/**
	*	public method to get the current Date and time stamps
	*
	* 	@return	String	Date/time stamp of the current datetime
	**/
	public String getDate()
	{
		Date today;
		String dateOut;
		String timeOut;
		DateFormat dateFormatter;
		DateFormat timeFormatter;
		
		dateFormatter = DateFormat.getDateInstance(DateFormat.LONG, new Locale("en","US"));
		timeFormatter = DateFormat.getTimeInstance(DateFormat.LONG, new Locale("en","US"));
		
		today = new Date();
		dateOut = dateFormatter.format(today);
		timeOut = timeFormatter.format(today);

		return dateOut + " " + timeOut;
	}
	
	public boolean inString(String sourceString, String value)
	{
		boolean result = false;
		int length = sourceString.length();
		
		for(int i=0; i<length; i++)
		{
			if(sourceString.charAt(i) == value.charAt(0))
			{
				result = true;
				i = length;
			}	
		}	
		
		return result;
	}
	
	public boolean inString(String sourceString, char value)
	{
		boolean result = false;
		int length = sourceString.length();

		for(int i=0; i<length; i++)
		{
			if(sourceString.compareToIgnoreCase(new Character(value).toString()) == 0)
			{
				result = true;
				i = length;
			}	
		}	
		
		return result;
	}		
			
	public int Write(String data, String filename) 
	{		
		int result=0;
		
		try 
		{
				FileOutputStream file = new FileOutputStream(filename);	//, true); //true means "append"
				PrintStream filePrintStream = new PrintStream(file);
				System.setOut(filePrintStream);
				//System.setErr(logfilePrintStream);
				System.out.println(data);
				result=1;
		} 
		catch (java.io.IOException exception) 
		{
			//printSystemOut("Write: Could not open " + filename);//+ logFileName + ". Make sure a log directory exists in your Aspiration installation directory.");
			//exception.printStackTrace();
		}
		
		return result;
	}	
	
    /**
  	*	Public setter method
  	*
  	**/
	public void setDebug(boolean value)
	{
		this.bDebug = value;
	}
	
	
	/**
	*	Public getter method
	*
	*	@return	int	size of the Gamut file.
	**/
	public boolean getDebug()
	{
		return this.bDebug;
	}
	
		
	/************************************************************************
	*	Public method to output debug data
	*
 	*	@param	value	String method name where the debug data originates
 	*	@param	value	String value for the debug data 	
	**************************************************************************/
	public void debug(boolean bDebug, String method, String value)
	{	
		if(bDebug)
		{
			//printSystemOut("METHOD: " + method + ": " + value); 
		}
	}
	
	public void debug(boolean bDebug, String method, List value)
	{	
		if(bDebug)
		{
			//printSystemOut("METHOD: " + method + ": " + value); 
		}
	}		

	public void debug(boolean bDebug, String method, int value)
	{	
		if(bDebug)
		{
			//printSystemOut("METHOD: " + method + ": " + value); 
		}
	}			
	
	/**
  	*	Private method that writes n carriage returns to a string
  	*
  	*	@param	lines	int number of carriage returns to output
  	**/
	public String nextline(int lines)
	{
		StringBuffer result = new StringBuffer();
		
		for (int i=0; i<lines; i++)
		{
			result.append(RN);
		}
		
		return result.toString();
	}
	
	
	/**
  	*	Private method to write one carriage return
  	**/
	public String nextline()
	{
		return RN;
	}	
	


	/**
  	*	public method that grabs all data nodes associated with a datafile tag. Method concatenates
  	*	all nodes after the first node and returns the result.
  	*
  	*	@param	lines	int number of carriage returns to output
  	**/	
	public String getDataNodes(List line)
	{
		int listSize = line.size();
		
		StringBuffer currentLineValues = new StringBuffer();
		
		for(int i=1; i<listSize; i++)
		{
			currentLineValues.append(line.get(i));
			
			// Put a space between values if not the last one
			if(i!=listSize-1)
			{
				currentLineValues.append(" ");
			}
		}
		
		return currentLineValues.toString();
	
	}
	
	/**
  	* Get a List of random Integers in the range 0 (inclusive) to N (exclusive).
  	*
  	* @param aUpperLimit must be greater than 0.
  	* @param aListSize must be greater than 0.
  	* @exception IllegalArgumentException if param does not comply.
  	*/ 
  	public List pickNumbers( int aUpperLimit, int aListSize ) 
  	{ 
  		if (aUpperLimit <= 0) 
  		{ 
  			throw new IllegalArgumentException( "UpperLimit must be positive: " + aUpperLimit);
    	} 
    	if (aListSize <= 0) 
    	{ 
    		throw new IllegalArgumentException( "Size of returned List must be greater than 0." );
    	} 
    	
    	// reuse one and the same Random object to generate a series 
    	// of random Integers 
    	
    	Random generator = new Random();
    	
    	List result = new ArrayList(); 
    	
    	for (int idx= 0; idx<aListSize; ++idx) 
    	{
      		result.add( new Integer(generator.nextInt(aUpperLimit)));
  
  		} 
  		
  		return result;
  	} 
	
	/**
  	* Get a List of random Integers in the range 0 (inclusive) to N (exclusive).
  	*
  	* @param aUpperLimit must be greater than 0.
  	* @param aListSize must be greater than 0.
  	* @exception IllegalArgumentException if param does not comply.
  	*/ 
  	public int pickNumber(int aUpperLimit) 
  	{ 
  		if (aUpperLimit <= 0) 
  		{ 
  			throw new IllegalArgumentException( "UpperLimit must be positive: " + aUpperLimit);
    	} 
    	
    	// reuse one and the same Random object to generate a series of random Integers 
    	
    	Random generator = new Random();
    	
    	return generator.nextInt(aUpperLimit);
  	} 	

    protected String toString(int value)
    {
    		return new Integer(value).toString();
    }
    
    public String getInputFilenameText()
    {
    		return inputFilenameText;
    }
	
	public void printSystemOut(String value)
	{
		boolean val=false;
		
		if(val)
			System.out.println(value);
	}	
}