import java.util.*;
import java.io.*;

public class Gamut extends Document
{					
	String File;							// Data file which contains piece specific info
	
	public int size;						// int size for number of Gamut squares in Gamut
	public int pieceSize;						// int size of piece as defined by SIZE in the data file
	public int systems;						// int number of systems for the current piece
	public String piecename;
	public String composer;
	public String poet;
	public String rights;
	public String software;
	public String encodingdate;
	public int process;
    public double length;					// combined length of all Gamut square durations (double)

	private Integer tempFlipsetNumber = new Integer(0);		// temp variables for creating list for coinflips
	private Integer tempSquareNumber = new Integer(0);

	public Hashtable jChingHashtable = new Hashtable();
	public Hashtable jChingDurationHashtable = new Hashtable();	
	public Hashtable jChingTranspositionHashtable = new Hashtable();
		
	public ArrayList systemClefs = new ArrayList();			// holds clefs for each system
	public ArrayList systemClefNumbers = new ArrayList();		// holds clefs for each system	
	public String systemClefString = new String();
	
	public ArrayList partNames = new ArrayList();	
	public String partNamesString = new String();
	
	GamutSquare CurrentGamutSquare = new GamutSquare();	// Current Gamut being processed	
	ArrayList GamutSquares = new ArrayList();			// Collection of GamutSquares in this Gamut
	
	public Hashtable Groups = new Hashtable();			// Group hashtable holding sub-arrays of gamutsquares in each group
	
	public Hashtable dynamicsHash = new Hashtable();
	public Hashtable defaultDynamicsHash = new Hashtable();
	
	public ArrayList dynamicsArray = new ArrayList();
	public String[] defaultDynamicsArray = {PPPP, PPP, PP, P, MP, MF, F, FF, FFF, FFFF, FP, FZ, SF, SFZ, SFFZ, SFP, EMPTY};
	public Hashtable dynamicsMIDIValues = new Hashtable();
	
    public String outputFilename;
    public String outputFilepath;
	public String outputType;
	
    public int chordCounter = 0;
    public int noteCounter = 0;
    
    // Array to hold notes with dotted values
    ArrayList dotArrayList = new ArrayList();
    
    Random generator = new Random();
    
	double activitymaximum = 0.0;
	double activityminimum = 100.0;		
	ArrayList activitylevels=new ArrayList();
    public int zones=0;
	
	
    /**
    * Public Constructor method: Init Piece object with no parameters
    **/
    public Gamut()
    {
			printSystemOut("GAMUT CONSTRUCTOR");
    		this.init();
    }
    
    
    /**
    *	Public Constructor method
    *
    *	@param filename	Init Piece object with a filename will IMPORT an existing Piece data file
    **/
    public Gamut(String filename)
    {
			printSystemOut("GAMUT CONSTRUCTOR w/filename");
    		this.setDebug(false);
    		this.init(filename);
    }	

    public Gamut(String filename, int zones)
    {
		printSystemOut("GAMUT CONSTRUCTOR zones = "+zones);
		this.setZones(zones);
		this.setDebug(false);
    	this.init(filename);
    }	

    /**
    *	Public Constructor method
    *
    *	@param debug	flag to turn on debug messages
    *	@param filename	Init Piece object with a filename will IMPORT an existing Piece data file
    **/
    public Gamut(boolean debug, String filename)
    {
    		this.setDebug(debug);
    		this.init(filename);		
    }
    
    public void init()
    {
			this.composer="testcomposername";
			
    		this.setDynamicMIDIValues();
    		this.setDefaultDynamicsHash();
    		this.setEncodingDate(this.getDate());
    }
    

    /**
    *	Public method to initialize class
    *
    *	@param	String name of file from which to read
    **/    
    public void init(String filename)
    {
        //this.GamutSquares = new ArrayList();   
		//this.init();
		this.setDynamicMIDIValues();							// should set this to read from a config file in current directory
		this.CurrentGamutSquare = new GamutSquare(getDebug());
		this.File = filename;
		this.size = loadGamut(filename); 	
    }
    
        
    /**
    *	Public method to load Gamut file
    *
    *	@param	String filepath to load for gamut file
    *	@return	int	size of gamut
    **/
	public int loadGamut(String filepath)
	{
		// Flag that signals the creation of a new Gamut square	
		boolean newGamutSquare = false;						
		
		// Open file, and get master List that holds Lists of all valid line values in each node (inherited code)
		List validLineValues = new ArrayList();
		validLineValues = GetFileValues(filepath);		
		int validLinesSize = validLineValues.size();
		
		// Step through master listarray...
		for(int i=0; i<validLinesSize; i++)
		{
			// Create a List of each line
			List singleValueArray = (List)validLineValues.get(i);
			
			newGamutSquare = processLine(singleValueArray);	// Pass each line list

			// create a new GamutSquare object and add it to the collection of Gamut objects			
			if(newGamutSquare)					// flag is set in processLine, to know when we're done a sqare		
			{	
				// Add the current gamut square to the arraylist of gamut squares
			    //	this.GamutSquares.add(this.CurrentGamutSquare);					
				
                // add duration length of current gamutSquare to Gamut length
                //  this.setLength(this.getLength() + this.CurrentGamutSquare.getLength());
				addGamutSquare(this.CurrentGamutSquare);
				
				// add activity level from this.CurrentGamutSquare to array
				setActivityLevels(this.CurrentGamutSquare.getActivityLevel());
			}
		}		
		
		// step through all GamutSquares in array and set which zone they belong to
		calculateZones();
		
		// Return count of Gamuts
		printSystemOut("Gamut.java : loadGamut() : this.GamutSquares.size() = " + this.GamutSquares.size());
		return this.GamutSquares.size();
	}
	
	public void calculateZones()
	{
		int count = this.GamutSquares.size();
		double activityrange = getActivityMaximum() - getActivityMinimum();
		double zoneSpread = activityrange/this.zones;

		printSystemOut("Gamut.java : calculateZones() : activity MIN = " + getActivityMinimum());
		printSystemOut("Gamut.java : calculateZones() : activity MAX = " + getActivityMaximum());
		printSystemOut("Gamut.java : calculateZones() : zones = " + this.zones);		
		printSystemOut("Gamut.java : calculateZones() : gscount = " + count);
		printSystemOut("Gamut.java : calculateZones() : zoneSpread = " + zoneSpread);		
		// for each gamutsquare
		for (int i=0; i<count; i++)
		{
			GamutSquare currentGamutSquare = getGamutSquare(i);
			double curActivity = currentGamutSquare.getActivityLevel();
			
			for(int j=0; j<this.zones; j++)
			{
				double currentMin = getActivityMinimum() + j*zoneSpread;
				double currentMax = zoneSpread + j*zoneSpread+getActivityMinimum();
				
				printSystemOut("currentMin = "+ currentMin);
				printSystemOut("currentMax = "+ currentMax);				
				
				if((curActivity<=currentMax)&&(curActivity>=currentMin))
				{
					((GamutSquare)this.GamutSquares.get(i)).setZone(j+1);
					printSystemOut("Gamut.java : zone value of " + curActivity + " for gs "+i+"="+((GamutSquare)this.GamutSquares.get(i)).getZone());
					j=this.zones;
				}
				else
				{
					printSystemOut("Gamut.java : zone value of " + curActivity + " for gs "+i+" does not fit");
				}
			}
		}
	}
	
	public int getZones()
	{
		return this.zones;
	}
	
	public void setZones(int value)
	{
		this.zones = value;
		printSystemOut("Gamut.java : setZones : this.zones = "+this.zones);
	}
	
	public void setActivityLevels(double activitylevel)
	{
		this.activitylevels.add(new Double(activitylevel));
		
		if(activitylevel>this.activitymaximum)
			setActivityMaximum(activitylevel);
		
		if(activitylevel<this.activityminimum)
			setActivityMinimum(activitylevel);
	}
	
	public void addGamutSquare(GamutSquare gs)
	{
		// Add the current gamut square to the arraylist of gamut squares
		this.GamutSquares.add(gs);					
		
        // add duration length of current gamutSquare to Gamut length
        this.setLength(this.getLength() + gs.getLength());		
	}
	
	/**
	*
	*	Use a CurrentGamutsquare object stored globally to store values until we're done with the current gamutsquare
	*
	**/
	public boolean processLine(List line)
	{
		boolean done = false;
		
		//printSystemOut(line.toString());
		
		// Check the first value of the line
		String firstvalue = line.get(0).toString();
		
		String choice = new String(firstvalue.toLowerCase());
        
		// First check to see if we're in the header section for SYSTEM and other piece level data
		if(choice.compareToIgnoreCase(SYSTEMS) == 0)
		{
			this.setSystems(this.getDataNodes(line));
		}
		else if(choice.compareToIgnoreCase(PIECENAME) == 0)
		{	
			this.setPieceName(this.getDataNodes(line));	
		}
		else if(choice.compareToIgnoreCase(COMPOSER) == 0)
		{
			this.setComposer(this.getDataNodes(line));
		}
		else if(choice.compareToIgnoreCase(POET) == 0)
		{
			this.setPoet(this.getDataNodes(line));		
		}
		else if(choice.compareToIgnoreCase(RIGHTS) == 0)
		{
			this.setRights(this.getDataNodes(line));		
		}
		else if(choice.compareToIgnoreCase(SOFTWARE) == 0)
		{
			this.setSoftware(this.getDataNodes(line));		
		}
		else if(choice.compareToIgnoreCase(ENCODINGDATE) == 0)
		{
			this.setEncodingDate(this.getDataNodes(line));		
		}
		else if(choice.compareToIgnoreCase(PARTS) == 0)
		{
			this.setPartNames(this.getDataNodes(line));
		}
		else if(choice.compareToIgnoreCase(SYSTEMCLEFS) == 0)
		{
			this.setSystemClefs((String)line.get(1));
		}
		else if(choice.compareToIgnoreCase(SIZE) == 0)
		{
			this.setPieceSize(this.getDataNodes(line));		
		}
		else if(choice.compareToIgnoreCase(PROCESS) == 0)
		{
			this.setProcess(this.getDataNodes(line));		
		}
        else if(choice.compareToIgnoreCase(OUTPUTFILENAME) ==0)
        {
			this.setOutputFilename(this.getDataNodes(line));		        
        }
        else if(choice.compareToIgnoreCase(OUTPUTFILEPATH) ==0)
        {
			this.setOutputFilepath(this.getDataNodes(line));		                
        }
        else if(choice.compareToIgnoreCase(OUTPUTTYPE) ==0)
        {
			this.setOutputType(this.getDataNodes(line));
        }		
		else if(choice.startsWith(COMMENTSTART))
		{
			// this is a comment, not the best way to capture yet
		}
		else if(choice.compareToIgnoreCase(GAMUTSQUARESTART) == 0)
		{
			// Since we're starting a new gamutsquare, clear CurrentGamutSquare
			this.CurrentGamutSquare = new GamutSquare();			
		}
		else if(choice.compareToIgnoreCase(NUMBER) == 0)					
		{		
			this.CurrentGamutSquare.setNumber((String)line.get(1));
			
			// Send NUMBER to buildjChingHash() - first set to temp var for Gamut, then wipe after used					
//			this.tempSquareNumber = new Integer((String)line.get(1));				
//			this.buildjChingHash();
		}
        else if(choice.compareToIgnoreCase(GROUP) ==0)
        {
			Integer value=new Integer((String)line.get(1));
			
			this.CurrentGamutSquare.setGroup(value);
			
			// Call Gamut buildGroups() method to add this gamutsquare number to a group
			this.buildGroups(value, this.CurrentGamutSquare.getNumber());
        }					
		else if(choice.compareToIgnoreCase(this.FLIPSETNUMBER) == 0)		
		{
			this.CurrentGamutSquare.setFlipsetNumber((String)line.get(1));				

			// Send flipsetNumber to buildjChingHash() - first set to temp var for Gamut, then wipe after used				
			this.tempFlipsetNumber = new Integer((String)line.get(1)); 				
			this.buildjChingHash();
		}
		else if(choice.compareToIgnoreCase(this.WEIGHTING) == 0)
		{			
			this.CurrentGamutSquare.setWeighting(new Integer((String)line.get(1)));
		}
		else if(choice.compareToIgnoreCase(this.METER) == 0)		
		{
			this.CurrentGamutSquare.setMeter(line);		
		}
		else if(choice.compareToIgnoreCase(this.CLEF) == 0)		
		{
			this.CurrentGamutSquare.setClef((String)line.get(1));					
		}		
		else if(choice.compareToIgnoreCase(this.KEYSIGNATURE) == 0)		
		{
			this.CurrentGamutSquare.setKeySignature((String)line.get(1));					
		}				
		else if(choice.compareToIgnoreCase(this.DURATIONS) == 0)		
		{
            // Pass in dotArrayList to method and return it as dotArrayList
			dotArrayList=this.CurrentGamutSquare.setDurations(line, dotArrayList);		
            
            // here, need to create one master list of all dotted notes
            // the note numbers need to be incremented by each set's total
            //dotArrayList=concatArrays(dotArrayList);
            
		}
		else if(choice.compareToIgnoreCase(this.NOTES) == 0)		
		{
            // return chordCounter number to keep track of unique chords across gamutSquares
//			this.chordCounter=
            
            int[] tempArray = this.CurrentGamutSquare.setNotes(line, chordCounter, dotArrayList, noteCounter);

            this.noteCounter=tempArray[0];
            this.chordCounter=tempArray[1];
		}
		else if(choice.compareToIgnoreCase(this.MARKUP) == 0)
		{
			this.CurrentGamutSquare.setMarkup((String)line.get(1));
		}
		else if(choice.compareToIgnoreCase(DYNAMICS)==0)
		{
			//printSystemOut("right before we process values: " +dynamicsHash.get(EMPTY));		
		}
		else if(choice.compareToIgnoreCase(this.END) == 0)		
		{
		
			// We've finished processing the current Gamut Square, now:
			//		1) copy duration values into Note objects
			//		2) set the current square into the Gamut collection
			
			// now set return Boolean to TRUE signifying that we have a new object to set into the collection 
			done = true;
		}			
		else		// check for Dynamics
		{
			//Integer curInteger = new Integer(line.get(1).toString());
			if(choice.compareToIgnoreCase(PPPP) == 0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));			
			}
			else if(choice.compareToIgnoreCase(PPP) == 0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}
			else if(choice.compareToIgnoreCase(PP) == 0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}
			else if(choice.compareToIgnoreCase(P)==0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}
			else if(choice.compareToIgnoreCase(MP)==0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}
			else if(choice.compareToIgnoreCase(MF)==0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}
			else if(choice.compareToIgnoreCase(F)==0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}
			else if(choice.compareToIgnoreCase(FF)==0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}
			else if(choice.compareToIgnoreCase(FFF)==0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}
			else if(choice.compareToIgnoreCase(FFFF)==0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}
			else if(choice.compareToIgnoreCase(EMPTY)==0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}
			else if(choice.compareToIgnoreCase(FP)==0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}
			else if(choice.compareToIgnoreCase(FZ)==0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}
			else if(choice.compareToIgnoreCase(SF)==0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}		
			else if(choice.compareToIgnoreCase(SFP)==0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}			
			else if(choice.compareToIgnoreCase(SFPP)==0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}			
			else if(choice.compareToIgnoreCase(SFZ)==0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}
			else if(choice.compareToIgnoreCase(SFFZ) ==0)
			{
				setDynamicValueArray(choice, new Integer(line.get(1).toString()));	
			}
			else
			{
				System.err.println(choice);
			}
		}					

		return done;

	}
	
	public void buildGroups(Integer group, Integer gamutsquare)
	{
		// if Integer group DNE, create array, add Integer gamutsquare to that array, and set that as hash group
		// -> this.Groups; is the hashtable we're using 
			printSystemOut("Gamut.java : buildGroups : group=" + group + ", gamutsquare#=" + gamutsquare);
			
			if(this.Groups.containsKey(group))
			{
				// check to see if existing group's arraylist contains this gamutsquare
				ArrayList values=(ArrayList)this.Groups.get(group);
				
				if(!values.contains(gamutsquare))
				{
					values.add(gamutsquare);
					this.Groups.put(group, values);
					printSystemOut("Gamut.java : buildGroups : added gamutsquare#=" + gamutsquare + " to group " + group);					
				}				
			}
			else
			{
				// add hash and array with single value of gamutsquare number
				ArrayList values=new ArrayList();
				values.add(gamutsquare);
				this.Groups.put(group, values);
				printSystemOut("Gamut.java : buildGroups : created group " + group + " and added gamutsquare#=" + gamutsquare + " to group");									
			}
	}
	
    /**
  	*	Public setter method
  	*
  	**/
	public void setFile(String value)
	{
		this.File = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Filename of the Gamut file.
	**/
	public String getFile()
	{
		return this.File;
	}
	
		
    /**
  	*	Public setter method
  	*
  	**/
	public void setSize(int value)
	{
		this.size = value;
	}

	
    /**
  	*	Public setter method
  	*
  	**/
	public void setSize(String value)
	{
		this.size = new Integer(value).intValue();
	}
	
	/**
	*	Public getter method
	*
	*	@return	int	size of the Gamut file.
	**/
	public int getSize()
	{
		return this.size;
	}

    /**
  	*	Public setter method
  	*
  	**/
	public void setPieceSize(String value)
	{
		this.pieceSize = new Integer(value).intValue();
	}
	
	/**
	*	Public getter method
	*
	*	@return	int	size of the Piece as defined in the data file.
	**/
	public int getPieceSize()
	{
		return this.pieceSize;
	}
	
    /**
  	*	Public setter method
  	*
  	**/
	public void setProcess(String value)
	{		
		this.process = new Integer(value).intValue();

	}
	
	public void setProcess(int value)
	{
		if(value>=0)
		{
			this.process = value;
		}
	}
	
	/**
	*	Public getter method
	*
	*	@return	int	size of the Piece as defined in the data file.
	**/
	public int getProcess()
	{
		return this.process;
	}
			
   /**
  	*	Public setter method
  	*
  	*	@param	value	String Name of the Piece.
  	**/
	public void setPieceName(String value)
	{
		this.piecename = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Name of the Piece
	**/
	public String getPieceName()
	{
		return this.piecename;
	}	
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Composer of the Piece.
  	**/
	public void setComposer(String value)
	{
		this.composer = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Composer of the Piece
	**/
	public String getComposer()
	{
		return this.composer;
	}		
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Poet of text for the Piece.
  	**/
	public void setPoet(String value)
	{
		this.poet = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Poet of text for the Piece
	**/
	public String getPoet()
	{
		return this.poet;
	}	
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Copyright data for the Piece.
  	**/
	public void setRights(String value)
	{
		this.rights = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Copyright data for the Piece
	**/
	public String getRights()
	{
		return this.rights;
	}
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Software-type data for the Piece.
  	**/
	public void setSoftware(String value)
	{
		this.software = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Software-type data for the Piece
	**/
	public String getSoftware()
	{
		return this.software;
	}		
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Encoding Date for the Piece.
  	**/
	public void setEncodingDate(String value)
	{
		this.encodingdate = value;
	}
	
	
	/**
	*	Public getter method
	*
	*	@return	String	Encoding date for the Piece
	**/
	public String getEncodingDate()
	{
		return this.encodingdate;
	}		
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	int number of Systems for the Piece.
  	**/
	public void setSystems(int value)
	{
		this.systems = value;
	}
	
	/**
	*	Public setter method
	*
	*	@param	value	int value of system for which to return clef String value
	**/
	public String getSystemClefs(int value)
	{
		return (String)this.systemClefs.get(value);
//		return (String)this.systemClefs.get(0);
	}
	
	/**
	*	Public setter method
	*
	*	@param	value	int value of system for which to return clef String value
	**/
	public String getSystemClefs(Integer value)
	{

		return 	this.getSystemClefs(value.intValue());
	}
	
		
	/**
	*	Public setter method
	*
	*	@param	value	int value of system for which to return clef number String value
	**/
	public String getSystemClefNumbers(int value)
	{
		return (String)this.systemClefNumbers.get(value);
	}
	
	/**
	*	Public setter method
	*
	*	@param	value	int value of system for which to return clef number String value
	**/
	public String getSystemClefNumbers(Integer value)
	{
		return 	this.getSystemClefNumbers(value.intValue());
	}
	
	public String getSystemClefsAsString()
	{
		int count = this.getSystems();
		String result = new String();
		
		for(int i=0;i<count;i++)
		{
			result=result+"/"+this.getSystemClefNumbers(i);
		}
	
		return result;
	}
			
    /**
  	*	Public setter method
  	*
  	*	@param	value	String holding / delineated Clefs for the Piece.
  	**/
	public void setSystemClefs(String value)
	{
		// systemClefs is an ArrayList that will hold Strings for each system's clef
		// need to step through input value and parse out the slashes

		this.systemClefString = value;
		
		printSystemOut(value);
		
		int stringSize = value.length();

		String tempString;
		Character tempCharacter;
		String clefValue = new String("");
		String clefNumber = new String("");
		
		String tempValue = new String();
		
		for(int i=0; i<stringSize; i++)
		{
			tempCharacter = new Character(value.charAt(i));
			tempString = new String(tempCharacter.toString());			
			
			if(tempString.compareToIgnoreCase(this.SLASH) != 0)
			{		
				//---------------------------------
				tempValue = tempValue+tempString;
			}
			else
			{
				tempString = tempValue;
				tempValue = "";
				//---------------------------------
				
				// Now check to see if we've entered Tenor, Alto, or Bass clefs
				if(tempString.compareToIgnoreCase(TREBLE) == 0)
				{
					clefValue = TREBLESIGN;
					clefNumber = TREBLELINE;
				}
				else if(tempString.compareToIgnoreCase(BASS) == 0)
				{
					clefValue = BASSSIGN;
					clefNumber = BASSLINE;
				}
				else if(tempString.compareToIgnoreCase(ALTO) == 0)
				{
					clefValue = ALTOSIGN;
					clefNumber = ALTOLINE;				
				}
				else if(tempString.compareToIgnoreCase(TENOR) == 0)
				{
					clefValue = TENORSIGN;
					clefNumber = TENORLINE;				
				}
				
				// add char to ArrayList								
				this.systemClefs.add(clefValue);
				this.systemClefNumbers.add(clefNumber);
			}	
		}
	}
	
	public String getSystemClefString()
	{
		return this.systemClefString;		
	}
	public String getPartNamesString()
	{
		return this.partNamesString;		
	}
	
	public void setPartNamesString(String value)
	{
		this.partNamesString = value;	
	}
	
	public void setPartNames(String value)
	{
		StringBuffer tempString = new StringBuffer();
		String currentChar = new String("");
		
		this.setPartNamesString(value);
		
		int charCounter = 0;
		
		int stringCount = value.length();
				
		for(int i=0; i<stringCount; i++)
		{		
			currentChar = new Character(value.charAt(i)).toString();
			
			if(currentChar.compareToIgnoreCase(SLASH) !=0) //(tempChar.compareToIgnoreCase(SPACE) != 0) && 
			{									
				// Add character to our string
				tempString.append(currentChar);
			}
			else	
			{
				// add tempString to partNames ArraylLst
				this.partNames.add(tempString.toString());
				
				tempString = new StringBuffer();
			}	
		}	
	}
	
	public String getPartNames(int index)
	{
		return (String)this.partNames.get(index);
	}
	
	public String getPartNamesAsString()
	{
		int count = this.partNames.size();
		String result = new String();
		
		for(int i=0;i<count;i++)
		{
			result=result+"/"+this.getPartNames(i);
		}
	
		return result;
	}	
    /**
  	*	Public setter method
  	*
  	*	@param	value	int number of Systems for the Piece.
  	**/
	public void setSystems(Integer value)
	{
		this.setSystems(value.intValue());
	}
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	int number of Systems for the Piece.
  	**/
	public void setSystems(String value)
	{
		this.setSystems(new Integer(value).intValue());
	}	
		
	/**
	*	Public getter method
	*
	*	@return	int	number of Systems for the Piece
	**/
	public int getSystems()
	{
		return this.systems;
	}			

    public void setLength(double value)
    {
        this.length = value;
    }
    
    public double getLength()
    {
        return this.length;
    }

	/**
	*	Public getter method
	*
	*	@return	GamutSquare object when passed unique number
	**/	
	public GamutSquare getGamutSquare(Integer value)
	{	
		int intValue = value.intValue();		
		GamutSquare result = (GamutSquare)this.GamutSquares.get(intValue);				
		return result;
	}
	
	
	/**
	*	Public getter method
	*
	*	@return	GamutSquare obeject when passed unique number
	**/	
	public GamutSquare getGamutSquare(int value)
	{
		//printSystemOut(" GAMUT.JAVA : getGamutSquare >>>>>> " + value);
		GamutSquare result = (GamutSquare)this.GamutSquares.get(value);		
		return result;
	}
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String file-name data for the Piece.
  	**/
	public void setOutputFilename(String value)
	{
		this.outputFilename = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	file-name data for the Piece
	**/
	public String getOutputFilename()
	{
		return this.outputFilename;
	}

    /**
  	*	Public setter method
  	*
  	*	@param	value	String Output file-path data for the Piece.
  	**/
	public void setOutputFilepath(String value)
	{
		this.outputFilepath = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Output file-path data for the Piece
	**/
	public String getOutputFilepath()
	{
		return this.outputFilepath;
	}	

    /**
  	*	Public setter method
  	*
  	*	@param	value	String Output type for the Piece.
  	**/
	public void setOutputType(String value)
	{
		this.outputType = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Output type for the Piece
	**/
	public String getOutputType()
	{
		return this.outputType;
	}	

	// call to set max, min, and arraylist of activity levels for all cells in this gamut
	public ArrayList getActivityLevels()
	{
		int count = GamutSquares.size();
		
		double lastvalue= 0.0;
		double currentvalue=0.0;
		double maximum=0.0;
		double minimum=0.0;
		
		ArrayList activityLevels=new ArrayList();
		
		for(int i=0; i<count; i++)
		{
			GamutSquare currentgamutsquare=(GamutSquare)GamutSquares.get(i);
			currentvalue=currentgamutsquare.getActivityLevel();
			
			if(currentvalue>maximum)
			{
				maximum=currentvalue;
			}
			else if(currentvalue<minimum)
			{
				minimum=currentvalue;
			}
			
			activityLevels.add(new Double(currentvalue));
		}
		
		setActivityMaximum(maximum);
		setActivityMinimum(minimum);
				
		return activityLevels;
	}
	
	public double getActivityMaximum()
	{
		return this.activitymaximum;
	}

	public double getActivityMinimum()
	{
		return this.activityminimum;
	}
	
	private void setActivityMaximum(double value)
	{
		this.activitymaximum = value;
	}
	
	private void setActivityMinimum(double value)
	{
		this.activityminimum = value;
	}
		
	// pass in P, 20 to give a 20 % weighting to p
	public void setDynamicValueArray(String dynamicMarking, Integer weighting)
	{		
		printSystemOut("Gamut.java : setDynamicValueArray : for "+dynamicMarking+ " " + weighting.toString());
		
		this.dynamicsHash.put(dynamicMarking, weighting);									
		
		printSystemOut("for "+dynamicMarking + " : " +this.dynamicsHash.get(dynamicMarking));
		
		int weightingInt = weighting.intValue();
				
		for(int i=1; i<=weightingInt; i++)
		{				
			this.dynamicsArray.add(dynamicMarking);								
		}		
	}
	
	
	
	public void setDefaultDynamicsHash()
	{
		String zero = new String("0");
		
		this.defaultDynamicsHash.put(PPPP, zero);
		this.defaultDynamicsHash.put(PPP, zero);
		this.defaultDynamicsHash.put(PP, zero);
		this.defaultDynamicsHash.put(P, zero);
		this.defaultDynamicsHash.put(MP, zero);
		this.defaultDynamicsHash.put(MF, zero);
		this.defaultDynamicsHash.put(F, zero);
		this.defaultDynamicsHash.put(FF, zero);
		this.defaultDynamicsHash.put(FFF, zero);
		this.defaultDynamicsHash.put(FFFF, zero);
		//this.dynamicsHash.put(SFPP, zero);
		this.defaultDynamicsHash.put(SFFP, zero);
		this.defaultDynamicsHash.put(SF, zero);
		this.defaultDynamicsHash.put(FP, zero);
		this.defaultDynamicsHash.put(SFP, zero);
		this.defaultDynamicsHash.put(FZ, zero);
		this.defaultDynamicsHash.put(SFZ, zero);
		this.defaultDynamicsHash.put(SFFZ, zero);
		this.defaultDynamicsHash.put(EMPTY, zero);
	}
	
	public String getDynamicValue()
	{
		String result = new String();
		
		int dynamicsArraySize = this.dynamicsArray.size();
		
		if(dynamicsArraySize>0)
		{
			result = (String)this.dynamicsArray.get(this.generator.nextInt(this.dynamicsArray.size()));
		}
		
		return result;		
	}	
	
	public String getDynamicMIDIValue(String value)
	{
		//printSystemOut("getDynamicMIDIValue("+value+") = ");

		Integer integerValue = (Integer)this.dynamicsMIDIValues.get(value.toUpperCase());
		
		
		//printSystemOut("getDynamicMIDIValue("+value+") = "+integerValue.toString());
		
		return integerValue.toString();		
	}
	
	public void setDynamicMIDIValues()
	{
		this.dynamicsMIDIValues.put(P, new Integer(54));
		this.dynamicsMIDIValues.put(PP, new Integer(40));
		this.dynamicsMIDIValues.put(PPP, new Integer(26));
		this.dynamicsMIDIValues.put(PPPP, new Integer(11));
		this.dynamicsMIDIValues.put(MP, new Integer(69));
		this.dynamicsMIDIValues.put(MF, new Integer(83));	
		this.dynamicsMIDIValues.put(F, new Integer(98));
		this.dynamicsMIDIValues.put(FF, new Integer(112));
		this.dynamicsMIDIValues.put(FFF, new Integer(127));
		this.dynamicsMIDIValues.put(FFFF, new Integer(141));		
		this.dynamicsMIDIValues.put(EMPTY, new Integer(0));
		this.dynamicsMIDIValues.put(SF, new Integer(-1));
		this.dynamicsMIDIValues.put(SFP, new Integer(-1));
		this.dynamicsMIDIValues.put(FP, new Integer(-1));
		this.dynamicsMIDIValues.put(FZ, new Integer(-1));		
		this.dynamicsMIDIValues.put(SFZ, new Integer(-1));
		this.dynamicsMIDIValues.put(SFFZ, new Integer(-1));
		this.dynamicsMIDIValues.put(SFFP, new Integer(-1));
	}
	
	
	
    
    
	// DON'T WANT TO USE THIS HASH TABLE ANY MORE. INSTEAD NEED TO USE AN ARRAYLIST; MAYBE CAN JUST USE ArrayList "GamutSquares"?
	
	
	// Method to add new Roll number (Cagian "666666" etc. jChing values as Keys and integer square numbers as values
	public void buildjChingHash()
	{	
		// Read each Gamut Square Number and jChing code and build a HashTable
		// based on the relationship between those values
		//printSystemOut("tempSquareNumber = " + tempSquareNumber +", tempFlipsetNumber = " + tempFlipsetNumber);

		if((this.tempSquareNumber.intValue() > 0) && (this.tempFlipsetNumber.intValue() > 0))
		{
			// Make new Hash entry for Roll -> Square Number
			this.jChingHashtable.put(this.tempFlipsetNumber, this.tempSquareNumber);
			
			this.tempSquareNumber = new Integer(0);
			this.tempFlipsetNumber = new Integer(0);
		}
	}	
}