import java.util.*;
import java.io.*;
import java.lang.reflect.*;

public class Lily extends Document
{	
	
	public String piecename;
	public String composer;
	public String poet;
	public String rights;
	public String software;
	public String encodingdate;
	public String[] instruments;
	public Piece currentPiece;

	public String lilyOutput;
	
	public int tabindex=0;
    public boolean firstMeasure=true;
    
    public boolean inChord = false;
    
    public int staffCounter=0;
    
    ArrayList processedChordNumbers = new ArrayList();
    
	StringBuffer header=new StringBuffer();
	StringBuffer parts=new StringBuffer();	
	
	String indentString="";
	String INDENT = "  ";
	int indentlevel=0;
	
	int lastnotechordvalue=0;
	
	String lastClef=new String();
	String lastTimeSignature=new String();
	public String lastDynamicValue = new String();
		
	/**
	*
	*	
	*	@param
	*	@value
	**/	
    public Lily()
    {
    
    }
	
    public Lily(boolean debug, Piece piece)
    {
       	this.setDebug(debug);       	
    	this.currentPiece = piece;    	
    	this.setPieceName(currentPiece.getPieceName());
    	this.setComposer(currentPiece.getComposer());
    	this.setPoet(currentPiece.getPoet());
    	this.setRights(currentPiece.getRights());
    	this.setSoftware(currentPiece.getSoftware());
    	this.setEncodingDate(currentPiece.getEncodingDate()); 	
    }	
	
    /**
    *
    * Takes a passed-in List of all data and converts to Lilypond
    *
    **/
	public int buildLily()
	{	
		StringBuffer result = new StringBuffer();		
	
//		this.header = writeHeader();
		result.append(writeHeader());
		
//		this.parts = writeParts();		
		result.append(writeParts(0));	// value of 0 means just write out all, other value is a duration

		setLilyOutput(result);
		currentPiece.setLilyOutput(result);
		
		// Print the result StringBuffer to file
        int value=this.Write(result.toString(), this.currentPiece.getOutputFilepath() + this.currentPiece.getOutputFilename());
		
		return value;
	}
	
	public int buildLily(boolean firstPage)
	{
		StringBuffer result = new StringBuffer();	
			
		if(firstPage)
		{
			// build First Page Header section
			result.append(writeFirstPageHeader());
		}
		else
		{
			// build header for next pages
			result.append(writeStandardHeader());
		}
		result.append(writeParts(0));	// value of 0 means just write out all, other value is a duration

		setLilyOutput(result);
		currentPiece.setLilyOutput(result);
		
		// Print the result StringBuffer to file
        int value=this.Write(result.toString(), this.currentPiece.getOutputFilepath() + this.currentPiece.getOutputFilename());
		
		return value;
	}
	
	public StringBuffer writeFirstPageHeader()
	{	
		return writeHeader();
	}
	
	public StringBuffer writeStandardHeader()
	{
		return new StringBuffer();
	}
	public void writeOutput(StringBuffer result)
	{
		this.Write(result.toString(), this.currentPiece.getOutputFilepath() + this.currentPiece.getOutputFilename());
	}
	
	
	// will generate 'numberofbeats' beats worth of music and set a global StringBuffer with that output value
	public void generateOutput(int numberofbeats)
	{
		StringBuffer result = new StringBuffer();
			
		result.append(writeHeader());
	}
	
	/**
	*
	*	
	*	@param
	*	@value
	**/	
	public StringBuffer writeHeader()
	{
		StringBuffer result = new StringBuffer();
		result.append("\\header {");
		result.append(this.tab(tabindex));		
		result.append(nextline());		
		result.append("title = "+quotes(this.piecename));
		result.append(nextline());
		result.append("composer = "+quotes(this.composer));
		result.append(nextline());
		result.append("copyright = "+quotes(this.rights));
		result.append(nextline());
		result.append("}");	
		result.append(nextline());
		//\override Staff.TimeSignature #'style = #'()
		return result;
	}
/**	
     \relative <<
       \new Staff {
         \time 3/4
         \clef treble
     
         e'2 d4 c2 b4 a8[ a]
         b[ b] g[ g] a2.
       }
       \new Staff {
          \clef bass
          c,,2 e4 g2.
          f4 e d c2.
       }
     >>
	 **/	
	 
	// Step through each System of the Piece and output each GamutSquare's data
	public StringBuffer writeParts(int count)
	{
		StringBuffer result = new StringBuffer();
		
//		result.append("\\relative <<");
//		result.append(nextline());		
				
		int systemCount = this.currentPiece.getSystems();
		        
        if(currentPiece.pieceGamut.getProcess() ==0)
        		systemCount=1;
        
		for(int i=0; i<systemCount; i++)
		{
			result.append(nextline());
			result.append("\\new Staff {");
			 
			// write part id tags
			result.append(nextline());								            
            firstMeasure=true;
            
			// For each part write out notes with writeNotes(systemNumber)
			result.append(this.writeNotes(i, count));

			result.append(nextline());				
			result.append("}");
			
			// Write Parts Closer		
			// ------------------
//            result.append(closeParts());            
		}		

		result.append(nextline());				
//		result.append(">>");
		result.append(nextline());		
		result.append("\\version \"2.6.0\"");	
		result.append(nextline());				
		return result;
	}	

	private StringBuffer writeNotes(int system, int count)
	{
		// if count=0 then just compute all (full read), otherwise it is a total duration which we want to calculate
		
				
		StringBuffer result = new StringBuffer();	
		
		// step through each system
		Staff currentStaff = (Staff)this.currentPiece.staffArray.get(system);
		
		// step through each Gamut Square in that system
		int gamutSquareCount = currentStaff.gamutSquares.size();

        double currentMeasureLength=0;
        int totalMeasureCount=0;
		
		for(int i=0; i<gamutSquareCount; i++)
		{			
			GamutSquare currentGamutSquare = (GamutSquare)currentStaff.getGamutSquare(i);
			int noteCount = currentGamutSquare.Notes.size();            
			this.lastnotechordvalue=0;
			
			printSystemOut("currentgamutSquare="+i);
			
			double totalduration=0;

			// track note durations to tell when we've crossed from 1 measure to the next
			int measurecount = currentGamutSquare.getMeasureCount();  
			int currentmeasure=0;
			double measureduration=0;
			
			// get total duration for this meter
			printSystemOut("measurecount="+measurecount+" , currentmeasure="+currentmeasure);
			
			boolean firstnote=true;
			
			String gsMarkup = new String();
			gsMarkup=getMarkup(currentGamutSquare);			
			
			// step through notes (i to noteCount)
			for(int j=0; j<noteCount; j++)
			{			
				Note currentnote=currentGamutSquare.getNote(j);

				String curMeter = currentGamutSquare.getMeterString(currentmeasure);
				printSystemOut("Lily.java : writeNotes : currentMeter String: "+curMeter);
				printSystemOut("Lily.java : writeNotes : lastTimeSignature: "+this.lastTimeSignature);
			
				if(firstnote)
				{					
					if(this.lastTimeSignature.compareToIgnoreCase(curMeter)!=0)
					{
						// write the time signature for each meter
						result.append(nextline());					
						result.append("\\time "+currentGamutSquare.getMeterString(currentmeasure));	
						result.append(nextline());
						this.lastTimeSignature=curMeter;								
					}
					
					if(this.lastClef.compareToIgnoreCase(currentGamutSquare.getClef())!=0)
					{		
						result.append("\\clef "+formatClef(currentGamutSquare.getClef())+nextline());
						this.lastClef = currentGamutSquare.getClef();						
					}
				}
				else
				{
					gsMarkup=new String();
				}
				
				if(measureduration<currentGamutSquare.getMeasureLength(currentmeasure))
				{
					firstnote=false;
					
					// only add duration if note is not a chord note or first note in a chord

                    // chordNoteNumber of 0 means no chord, value of 1 means first note in chord
                    if(currentnote.getChordNoteNumber()<2)
                    {
//                      curDuration = currentNote.getDuration()/lowestSubdivision; 
						measureduration=measureduration+currentnote.getDuration();                                
                    }

				}
				else // if we've reached the measureduration
				{
					currentmeasure++;
					result.append(nextline());					
					result.append(nextline());
					
					printSystemOut("Lily.java : writeNotes : currentMeter String: "+curMeter);
					printSystemOut("Lily.java : writeNotes : lastTimeSignature: "+this.lastTimeSignature);
					
					if(this.lastTimeSignature.compareToIgnoreCase(curMeter)!=0)
//					if(((currentGamutSquare.getMeterString(currentmeasure)).compareToIgnoreCase(this.lastTimeSignature))!=0)
					{
						result.append("\\time "+currentGamutSquare.getMeterString(currentmeasure));
						result.append(nextline());	
						this.lastTimeSignature=currentGamutSquare.getMeterString(currentmeasure);
					}
					
					if(this.lastClef.compareToIgnoreCase(currentGamutSquare.getClef())!=0)
					{
						result.append("\\clef "+formatClef(currentGamutSquare.getClef())+nextline());
						this.lastClef = currentGamutSquare.getClef();
					}
						
					measureduration=currentnote.getDuration();					
				}				
				
				result.append(writeNote(currentnote)+gsMarkup);								
			}							
		}
		
		return result;		
	}
	
	private String getMarkup(GamutSquare gs)
	{
		String value = gs.getMarkup();
		String result=new String();
		
		if(value.compareToIgnoreCase("pizz")==0)
		{
			result="^\\markup { \\small \"pizz\" }";
		}
		else if(value.compareToIgnoreCase("arco")==0)
		{
			result="^\\markup { \\small \"arco\" }";
		}
		else if(value.compareToIgnoreCase("accel")==0)
		{
			result="^\\markup { \\small \"accel.\" }";
		}
		else if(value.compareToIgnoreCase("decel")==0)
		{
			result="^\\markup { \\small \"decel.\" }";
		}
		
		return result;
	}
	
	
	public String formatClef(String clef)
	{
		String result = new String();
		
		if(clef.compareToIgnoreCase("TR")==0)
		{
			result="treble";
		}
		else if(clef.compareToIgnoreCase("BA")==0)
		{
			result="bass";
		}
		else if(clef.compareToIgnoreCase("AL")==0)
		{
			result="alto";
		}
		else if(clef.compareToIgnoreCase("TE")==0)
		{
			result="tenor";
		}		
		else if(clef.compareToIgnoreCase("SO")==0)
		{
			result="soprano";
		}		
		
//		this.lastClef=clef;
		
		return result;
	}
	
	public String writeNote(Note currentnote)
	{
		String result = new String();
		String prepad = " ";
		String postpad = " ";
		
		boolean chordstart, chordend = false;
		
		int currentnotechordvalue = currentnote.getChordNoteNumber();
		
		// end tag for previous chord
		if((this.lastnotechordvalue>0) && (currentnotechordvalue<2))
		{			
			result=result+">> ";
			prepad="";
		}
		else
		{
			prepad=" ";
		}
		
		printSystemOut("lastnotechordvalue="+this.lastnotechordvalue);
		printSystemOut("currentnotechordvalue="+currentnotechordvalue);
		
		// check for chord
		if(currentnotechordvalue==1)
		{
			result=result+" <<";
			postpad = "";
		}
		else
		{
			postpad = " ";
		}
		
		String value=currentnote.getAccidental();
		String accidentalString = new String();
		
		if(value == "1") // sharp
		{
			accidentalString = "is";
		}
		else if(value == "2") //doublesharp
		{
			accidentalString = "isis";
		}        
		else if(value == "-1")
		{
			accidentalString = "es";
		}
		else if(value == "-2")
		{
			accidentalString = "eses";		
		}        
		
		int dot=currentnote.getDot();
		String dots = new String();
		
		for(int d=0; d<dot;d++)
		{
			dots=dots+".";
		}
		
		String durationType = currentnote.getType();
		String duration = new String();
		
		if(durationType == "whole")
		{
			duration="1";
		}
		else if(durationType == "half")
		{
			duration="2";
		}	
		else if(durationType == "quarter")
		{
			duration="4";
		}	
		else if(durationType == "eighth")
		{
			duration="8";
		}	
		else if(durationType == "sixteenth")
		{
			duration="16";
		}	
		else if(durationType == "thirty-second")
		{
			duration="32";
		}	
		
		int articulationcount = currentnote.articulations.size();
		String articulation = new String();		
		
		for (int i=0; i<articulationcount; i++)
		{
			String notearticulation = currentnote.getArticulation(i);

			printSystemOut("notearticulation="+notearticulation);
			
			if(notearticulation.compareToIgnoreCase("staccato")==0)
			{
				articulation = articulation+"-.";
			}
			if(notearticulation.compareToIgnoreCase("tenuto")==0)
			{
				articulation = articulation+"--";
			}		
			if(notearticulation.compareToIgnoreCase("accent")==0)
			{
				articulation = articulation+"->";
			}		
		}
		
		//printSystemOut("*** currentNote rest? " + currentnote.getRest());
		
		// get note letter
		result=result + prepad+currentnote.getNoteValue().toLowerCase() + accidentalString+this.getLilyOctave(currentnote.getOctaveNumber()) + duration +dots + articulation + setDynamic(currentnote) + postpad;
						
		this.lastnotechordvalue = currentnotechordvalue;
		
		return result;
	}
	
    public String setDynamic(Note currentnote)
    {
    		String result = new String();
    		
    		String value = currentPiece.pieceGamut.getDynamicValue();
    		
    		if(value!=NULL)
    		{    			
    			// if the cur dynamic isn't the same as the last AND its not EMPTY and we're not on the nth note of a chord...
    			if(value.compareToIgnoreCase(this.lastDynamicValue)!=0 && value.compareToIgnoreCase(EMPTY)!=0 && !this.inChord)
    			{
					result="\\"+value;
					
					// don't write lastValue for -1's like SFZ, they don't carry past the note
					this.lastDynamicValue = value;
    			}    			    			
    		}
    		
        return result;
    }
	
	public String quotes(String value)
	{
		return "\""+value+"\"";
	}
	
	/** GETTERS AND SETTERS **/
	//--------------------------
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Name of the Piece.
  	**/
	public void setPieceName(String value)
	{
		this.piecename = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Name of the Piece
	**/
	public String getPieceName()
	{
		return this.piecename;
	}	
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Composer of the Piece.
  	**/
	public void setComposer(String value)
	{
		this.composer = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Composer of the Piece
	**/
	public String getComposer()
	{
		return this.composer;
	}		
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Poet of text for the Piece.
  	**/
	public void setPoet(String value)
	{
		this.poet = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Poet of text for the Piece
	**/
	public String getPoet()
	{
		return this.poet;
	}	
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Copyright data for the Piece.
  	**/
	public void setRights(String value)
	{
		this.rights = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Copyright data for the Piece
	**/
	public String getRights()
	{
		return this.rights;
	}
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Software-type data for the Piece.
  	**/
	public void setSoftware(String value)
	{
		this.software = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Software-type data for the Piece
	**/
	public String getSoftware()
	{
		return this.software;
	}		
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Encoding Date for the Piece.
  	**/
	public void setEncodingDate(String value)
	{
		this.encodingdate = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Encoding date for the Piece
	**/
	public String getEncodingDate()
	{
		return this.encodingdate;
	}			

	public String getClefSign(Integer value)
	{
		// For the current staff, get the appropriate clef sign
		// these are marked in data file as "CLEFS T/T/A/G" etc.
		// stored as gamut.getSystemClefs() which is an arraylist
		// get this.currentPiece.pieceGamut.getSystemClefs(value)
		return this.currentPiece.pieceGamut.getSystemClefs(value);
	}
		
	public String getClefSign(int value)
	{
		// For the current staff, get the appropriate clef sign
		// these are marked in data file as "CLEFS T/T/A/B" etc.
		// stored as gamut.getSystemClefs() which is an arraylist
		// get this.currentPiece.pieceGamut.getSystemClefs(value)
		
		return this.currentPiece.pieceGamut.getSystemClefs(value);
	}
	
	public String getClefNumber(int value)
	{
		return this.currentPiece.pieceGamut.getSystemClefNumbers(value);
	}

	public String getClefNumber(Integer value)
	{
		return this.getClefNumber(value);
	}
	
	public String getLilyOutput()
	{
		return lilyOutput;		
	}
	
	public void setLilyOutput(String value)
	{
		this.lilyOutput = value;
	}

	public void setLilyOutput(StringBuffer value)
	{
		this.lilyOutput = value.toString();
	}	

	public String tab(int value)
	{
		StringBuffer result = new StringBuffer();
		
		for(int i=0; i<= value; i++)
		{
			result = result.append(TAB);	
		}	

		this.tabindex = value;
				
		return result.toString();
	}
	
    public void setTab(int value)
    {
        this.tabindex=value;
    }	
	
	public String makeString(Integer value)
	{
		return value.toString();
	}
	
	public String makeString(int value)
	{
		Integer tempInteger = new Integer(value);

		return tempInteger.toString();		
	}

	public String makeString(double value)
	{
		Double tempDouble = new Double(value);

		return tempDouble.toString();		
	}
	
	public String getLilyOctave(int value)
	{
		String result=new String();

		if(value==8)
		{
			result="''''";
		}		
		if(value==7)
		{
			result="'''";
		}		
		if(value==6)
		{
			result="''";
		}		
		if(value==5)
		{
			result="'";
		}		
		if(value==4)
		{
			result="";
		}
		else if(value==3)
		{
			result=",";			
		}
		else if(value==2)
		{
			result=",,";			
		}
		else if(value==1)
		{
			result=",,,";			
		}

		return result;
	}

	public String indent()
	{
		return this.indentString;
	}

	public String indent(String value)
	{
		String localindentString="";
		
		if(value=="+")
		{
			this.indentlevel=this.indentlevel+1;
		}
		else if(value=="-")
		{
			this.indentlevel=this.indentlevel-1;
					
		}
		
		for(int i=0; i<this.indentlevel; i++)
		{
			localindentString=localindentString+INDENT;
		}
		
		return localindentString;
	}
}