import java.util.*;
import java.io.*;
import java.lang.reflect.Array;

import com.sun.rsasign.g;

public class Piece extends Document
{
	String File;	// Data file which contains piece specific info

	public String piecename;
	public String composer;
	public String poet;
	public String rights;
	public String software;
	public String encodingdate;
	public int process;						// selection process to be used for this piece
	public double length;					// length in beats or decimal fractions of beats
	public String[] instruments;
	public int systems;						// number of systems to create for the current piece
	public Gamut pieceGamut;
	public ArrayList staffArray = new ArrayList();			// ArrayList of the Staffs in this piece
	public ArrayList gamutNoteArray = new ArrayList();		// ArrayList of the coin flips which form the Gamut square array
	public ArrayList gamutDurationArray = new ArrayList();		// ArrayList of the duration gamut squares
	public ArrayList gamutTranspositionArray = new ArrayList();	// ArrayList of the transposition gamut squares	
	public String outputType;				// LILYPOND, MUSICXML
	public String outputFilename;
    public String outputFilepath;
    
    public String xmlOutput = new String();
    public String lilyOutput = new String();
   //inputFilenameText inherited from Document
    
	public Hashtable jChingNoteHashtable;
	public Hashtable jChingDurationHashtable;	
    
	Random generator = new Random();
    
	public int zones=0;
	public double buildincrement=0;
	
    /**
    *	Public Constructor method
    *
    *	@param filename	Init Piece object with a filename will IMPORT an existing Piece data file
    **/
    public Piece(boolean debug, String filename)
    {
       	this.setDebug(debug);    	       	           	       	       	    
        this.setGamut(new Gamut(this.getDebug(), filename));
        this.init(filename);
    }	

    public Piece(String filename, int zones)
    {
        this.setGamut(new Gamut(filename, zones));
        this.init(filename);
    }
	    
    public Piece(String filename)
    {
        this.setGamut(new Gamut(filename));
        this.init(filename);
    }
    
    // Constructor for empty Piece object
    public Piece()
    {    
    		this.setGamut(new Gamut());
    }
    
    // populate an empty Piece object here
    public void loadPiece(String filename)
    {
    		setGamut(new Gamut(filename));
    		init(filename);    		
    }
    
    public void init(String filename)
    {
		this.File = filename;
		this.systems = this.pieceGamut.getSystems();
		this.piecename = this.pieceGamut.getPieceName();
		this.rights = this.pieceGamut.getRights();
		this.composer = this.pieceGamut.getComposer();
		this.poet = this.pieceGamut.getPoet();
		this.encodingdate = this.pieceGamut.getEncodingDate();
		this.software = this.pieceGamut.getSoftware();
		this.length = this.pieceGamut.getPieceSize();					// from the SIZE var in the .gam file
		this.jChingNoteHashtable = this.pieceGamut.jChingHashtable;		// Problem with copying Hashtable. figure out how to
		this.process = this.pieceGamut.getProcess();					// new 5.16.05
		
        this.outputFilename = this.pieceGamut.getOutputFilename();
        this.outputFilepath = this.pieceGamut.getOutputFilepath();
        this.outputType = this.pieceGamut.getOutputType();
		
		this.jChingDurationHashtable = this.pieceGamut.jChingDurationHashtable;  	// do we need duration hash?	
		
		//this.pieceGamut.setDynamicMIDIValues();
    }
    
    
    
	// Output the full Gamut set without any modifications
	public void outputGamut()
	{
		Staff currentStaff = new Staff(1);
		
		int tempGamutNumber = 0;
		int tempDurationNumber = 1;			
		int tempTranspositionNumber = 1;						
					
		GamutSquare currentGamutSquare;
								
		double currentGamutSquareLength = 0;
		double currentStaffLength = 0;
		
		// Number of Gamut squares in our gamut
		int gamutSize = this.pieceGamut.GamutSquares.size();           
		
		// Step through each gamut square
		for(int i=0; i<gamutSize; i++)
		{				
			currentGamutSquare = (GamutSquare)this.pieceGamut.getGamutSquare(new Integer(i));
							
			// Add the currentGamut Square to the Staff object while setting the duration and 
            // transposition multipliers at the same time
			// Also adds the length of the gamutsquare to the currentStaff
			currentStaff.addGamutSquare(currentGamutSquare, tempDurationNumber, tempTranspositionNumber);					
		}
		
		// Add currentStaff object to the Piece object's staffArray() ArrayList
		this.staffArray.add(currentStaff);			
	}
	
	public ArrayList incrementalBuild()
	{        
		ArrayList staves=new ArrayList();
		// Create Staff object for each system (one for each system)
		for(int i=0; i<this.systems; i++)
		{
			Staff currentStaff = new Staff(i+1);
			staves.add(currentStaff);
		} 
		
		return staves;
	}

	// Main computational method that will use Cage-ian iChing coin-flip techniques to generate
	// a gamut-square based structure for the piece. Multiple rows of gamut squares (with varying
	// durations and transpositions) will be generated using the coin-flip object
	public void buildSystems()
	{					
        // Init random number generator
        int currentRandom;
        //Random generator = new Random();              
        
        int tempGamutNumber = 0;
        int tempDurationNumber = 1;			
        int tempTranspositionNumber = 1;						
        
        Integer NoteGamutCoinFlips;
        Integer DurationGamutCoinFlips;
        Integer TranspositionGamutCoinFlips;
        Integer tempInteger;			

        GamutSquare currentGamutSquare;
                                
        double currentGamutSquareLength = 0;
        double currentStaffLength = 0;
        
        // GET DURATION MODIFIERS AND TRANSPOSITION MODIFIERS
        
        // HERE MAKE DIFFERENTIATION BETWEEN PROCESSES

        // HERE we check the process variable of the GAMUT and based on that value, use different selection criteria
        // for cells. The Flipset concept is no longer useful (if it ever was)
        //	0 = no processing output=input
        //	1 = completely random, cells can appear more than once - no weighting
        //	2 = degrading random, cells only appear once - no weighting
        //	3 = completely random, cells can appear more than once - with weighting
        //	4 = degrading random, cells only appear once - with weighting
        
        //  FOR PROCESSES, CHECK DESIRED PIECE LENGTH AND INCORPORATE THIS CHOICE WHERE POSSIBLE
        // (will be wierd when there isn't enough data to generate a certain size)
        // while(currentStaff.getLength() < this.size)

        int tempCellcount;
        int tempWeight;
        int currentProcess;
        int gamutSize;
        int currentCount;
        ArrayList weightedArray;
        ArrayList unweightedArray;        
        int tempSize;
        int tempValue;
        
		// Create Staff object for each system (one for each system)
		for(int i=0; i<this.systems; i++)
		{
			Staff currentStaff = new Staff(i+1);			 
			currentProcess = this.pieceGamut.process;
			gamutSize = this.pieceGamut.GamutSquares.size();
			
			if(currentProcess==0)
			{
				// step through each cell in order, 1 through end
				
				// CASE 1: Simple cells selection in order
				for(int k=0; k<gamutSize; k++)
				{
					currentCount = k;
					currentGamutSquare = (GamutSquare)this.pieceGamut.getGamutSquare(currentCount);
                    
                    // Need to do something with tempDuration/Transposition numbers
					currentStaff.addGamutSquare(currentGamutSquare, tempDurationNumber, tempTranspositionNumber);
					//this.gamutNoteArray.add(NoteGamutCoinFlips);	// DOES THIS DO ANYTHING?
                    
                    // this is essentially a flat-reading of the file; only process one staff
                    i=this.systems;
				}				
			}
			else if (currentProcess==1)
			{
                // For now, this is a crude way of checking length. What we need to do is to check before adding a cell whether or not it will fit the length. If the selected cell is only rests, then it's ok, we rest until the end. If the cell length added to the current length will be over the limit, we start checking whether or not we can add individual notes to fill up the rest of the piece size. If not, we rest out. If so, we only use those notes of the cell which will fill up the length - this has to take into account duration modifiers.
                
                // Actually, this kind of calculation only needs to be applied in STAFF during cell output, when a new measure is reached, then a new measure is written.
                
                while(currentStaff.getLength() < this.length)
                {
                    // Cells will be selected randomly and added to the piece until the desired length in beats is reached. 
                    // Weighting is ignored.                
                    currentRandom = generator.nextInt(gamutSize);
                    
                    // Get random value from list between 0 and (gamutSize-1)                
                    currentGamutSquare = (GamutSquare)this.pieceGamut.getGamutSquare(currentRandom);
                    currentStaff.addGamutSquare(currentGamutSquare, tempDurationNumber, tempTranspositionNumber);
                }
			}
			else if (currentProcess==2)
			{
				// randomly select cells, ignore weighting, then remove selected cell values from list
                //-----------------------------------------------------------
                
                // Make simple array of all cell numbers including multiple instances of each as dictated by weightings
                unweightedArray = new ArrayList();
                
                tempCellcount = this.pieceGamut.getSize();                
                
                // Step through each GamutSquare, check their weighting value, and add n-instances
                // of that cell to the array. We will then later select indices from that array
                // randomly and use their corresponding values (cell numbers) as selected GamutSquares
                for(int k=0; k<tempCellcount;k++)
                {                    
                    unweightedArray.add(new Integer(k));                                            
                }
                
                // Use length of array - 1 (0 based) as uBound for random generator                
                while(currentStaff.getLength() < this.length && unweightedArray.size() > 0)
                {
                    currentRandom = generator.nextInt(unweightedArray.size());                     
                    tempValue = ((Integer)unweightedArray.get(currentRandom)).intValue();
                                       
                     // Get random value's corresponding cell number from "weightedArray"                    
                    currentGamutSquare = (GamutSquare)this.pieceGamut.getGamutSquare(tempValue);
                    
                    currentStaff.addGamutSquare(currentGamutSquare, tempDurationNumber, tempTranspositionNumber);
                    
                    // remove just-selected cell from ArrayList.
                    unweightedArray.remove(currentRandom);
                }

			}
			else if (currentProcess==3)
			{
				// randomly select cells, use selection table with weightings
                //-----------------------------------------------------------
                
                // Make simple array of all cell numbers including multiple instances of each as dictated by weightings
                weightedArray = new ArrayList();
                
                tempCellcount = this.pieceGamut.getSize();                
                
                // Step through each GamutSquare, check their weighting value, and add n-instances
                // of that cell to the array. We will then later select indices from that array
                // randomly and use their corresponding values (cell numbers) as selected GamutSquares
                for(int k=0; k<tempCellcount;k++)
                {
                    tempWeight = this.pieceGamut.getGamutSquare(k).getWeighting();
                    
                    // loop "tempWeight" number of times to put multiple instances in array
                    for(int j=0; j<tempWeight; j++)
                    {
                        // Add one value for each weighting (so if weighting = 3 then we add that cell 3 times) 
                        weightedArray.add(new Integer(k));                        
                    }
                }
                
                // Use length of array - 1 (0 based) as uBound for random generator
                tempSize = weightedArray.size();
                
                // Same code as above but using tempSize
                while(currentStaff.getLength() < this.length)
                {
                    currentRandom = generator.nextInt(tempSize);                     
                    tempValue = ((Integer)weightedArray.get(currentRandom)).intValue();
                                       
                     // Get random value's corresponding cell number from "weightedArray"                    
                    currentGamutSquare = (GamutSquare)this.pieceGamut.getGamutSquare(tempValue);
                    
                    currentStaff.addGamutSquare(currentGamutSquare, tempDurationNumber, tempTranspositionNumber);
                }
                                
			}
			else if (currentProcess==4)
			{
				// randomly select cells, use weightings table, remove cells from list when selected
                //-----------------------------------------------------------
                
                // Make simple array of all cell numbers including multiple instances of each as dictated by weightings
                weightedArray = new ArrayList();
                
                tempCellcount = this.pieceGamut.getSize();                
                
                // Step through each GamutSquare, check their weighting value, and add n-instances
                // of that cell to the array. We will then later select indices from that array
                // randomly and use their corresponding values (cell numbers) as selected GamutSquares
                for(int k=0; k<tempCellcount;k++)
                {
                    tempWeight = this.pieceGamut.getGamutSquare(k).getWeighting();
                    
                    // loop "tempWeight" number of times to put multiple instances in array
                    for(int j=0; j<tempWeight; j++)
                    {
                        // Add one value for each weighting (so if weighting = 3 then we add that cell 3 times) 
                        weightedArray.add(new Integer(k));                        
                    }
                }
                                
                // Same code as above but using tempSize
                while(currentStaff.getLength() < this.length && weightedArray.size() > 0)
                {
                    currentRandom = generator.nextInt(weightedArray.size());                     
                    tempValue = ((Integer)weightedArray.get(currentRandom)).intValue();
                                       
                     // Get random value's corresponding cell number from "weightedArray"                    
                    currentGamutSquare = (GamutSquare)this.pieceGamut.getGamutSquare(tempValue);
                    
                    currentStaff.addGamutSquare(currentGamutSquare, tempDurationNumber, tempTranspositionNumber);
                    
                    // remove just-selected cell from ArrayList. Do we have to shift the others??
                    weightedArray.remove(currentRandom);
                }                
			}				

			// Add currentStaff object to the Piece object's staffArray() ArrayList
			this.staffArray.add(currentStaff);
				
		}
			
	}

	
	public ArrayList buildSystems(ArrayList staves)//Staff currentStaff)
	{					
		// build result ArrayList for output
		ArrayList resultStaves=new ArrayList();
        
		// Init random number generator
        int currentRandom;
        //Random generator = new Random();              
        
        int tempGamutNumber = 0;
        int tempDurationNumber = 1;			
        int tempTranspositionNumber = 1;						
        
        Integer NoteGamutCoinFlips;
        Integer DurationGamutCoinFlips;
        Integer TranspositionGamutCoinFlips;
        Integer tempInteger;			

        GamutSquare currentGamutSquare;
                                
        double currentGamutSquareLength = 0;
        double currentStaffLength = 0;
        
        // GET DURATION MODIFIERS AND TRANSPOSITION MODIFIERS
        
        // HERE MAKE DIFFERENTIATION BETWEEN PROCESSES

        // HERE we check the process variable of the GAMUT and based on that value, use different selection criteria
        // for cells. The Flipset concept is no longer useful (if it ever was)
        //	0 = no processing output=input
        //	1 = completely random, cells can appear more than once - no weighting
        //	2 = degrading random, cells only appear once - no weighting
        //	3 = completely random, cells can appear more than once - with weighting
        //	4 = degrading random, cells only appear once - with weighting
        
        //  FOR PROCESSES, CHECK DESIRED PIECE LENGTH AND INCORPORATE THIS CHOICE WHERE POSSIBLE
        // (will be wierd when there isn't enough data to generate a certain size)
        // while(currentStaff.getLength() < this.size)

        int tempCellcount;
        int tempWeight;
        int currentProcess;
        int gamutSize;
        int currentCount;
        ArrayList weightedArray;
        ArrayList unweightedArray;        
        int tempSize;
        int tempValue;
        
		// staves = ArrayList of Staff objects passed in
		//		process each staff here
		
		int staffcount = staves.size();

		// Create Staff object for each system (one for each system)
		for(int i=0; i<staffcount; i++)
		{
			Staff currentStaff = (Staff)staves.get(i);				//new Staff(i+1);			 
			
			// set desiredlength to the number of beats currently in the currentStaff + buildincrement
			double desiredlength = currentStaff.getLength() + this.buildincrement;
			
			currentProcess = this.pieceGamut.process;
			gamutSize = this.pieceGamut.GamutSquares.size();
			
			if(currentProcess==0)
			{
				// step through each cell in order, 1 through end
				
				// CASE 1: Simple cells selection in order
				for(int k=0; k<gamutSize; k++)
				{
					currentCount = k;
					currentGamutSquare = (GamutSquare)this.pieceGamut.getGamutSquare(currentCount);
                    
                    // Need to do something with tempDuration/Transposition numbers
					currentStaff.addGamutSquare(currentGamutSquare, tempDurationNumber, tempTranspositionNumber);
					//this.gamutNoteArray.add(NoteGamutCoinFlips);	// DOES THIS DO ANYTHING?
                    
                    // this is essentially a flat-reading of the file; only process one staff
                    i=this.systems;
				}				
			}
			else if (currentProcess==1)
			{
                // For now, this is a crude way of checking length. What we need to do is to check before adding a cell whether or not it will fit the length. If the selected cell is only rests, then it's ok, we rest until the end. If the cell length added to the current length will be over the limit, we start checking whether or not we can add individual notes to fill up the rest of the piece size. If not, we rest out. If so, we only use those notes of the cell which will fill up the length - this has to take into account duration modifiers.
                
                // Actually, this kind of calculation only needs to be applied in STAFF during cell output, when a new measure is reached, then a new measure is written.
                
                while(currentStaff.getLength() < desiredlength)  //this.length)
                {
                    // Cells will be selected randomly and added to the piece until the desired length in beats is reached. 
                    // Weighting is ignored.                
                    currentRandom = generator.nextInt(gamutSize);
                    
                    // Get random value from list between 0 and (gamutSize-1)                
                    currentGamutSquare = (GamutSquare)this.pieceGamut.getGamutSquare(currentRandom);
                    currentStaff.addGamutSquare(currentGamutSquare, tempDurationNumber, tempTranspositionNumber);
                }
			}
			else if (currentProcess==2)
			{
				// randomly select cells, ignore weighting, then remove selected cell values from list
                //-----------------------------------------------------------
                
                // Make simple array of all cell numbers including multiple instances of each as dictated by weightings
                unweightedArray = new ArrayList();
                
                tempCellcount = this.pieceGamut.getSize();                
                
                // Step through each GamutSquare, check their weighting value, and add n-instances
                // of that cell to the array. We will then later select indices from that array
                // randomly and use their corresponding values (cell numbers) as selected GamutSquares
                for(int k=0; k<tempCellcount;k++)
                {                    
                    unweightedArray.add(new Integer(k));                                            
                }
                
                // Use length of array - 1 (0 based) as uBound for random generator                
                while(currentStaff.getLength() < desiredlength && unweightedArray.size() > 0)// < this.length && unweightedArray.size() > 0)
                {
                    currentRandom = generator.nextInt(unweightedArray.size());                     
                    tempValue = ((Integer)unweightedArray.get(currentRandom)).intValue();
                                       
                     // Get random value's corresponding cell number from "weightedArray"                    
                    currentGamutSquare = (GamutSquare)this.pieceGamut.getGamutSquare(tempValue);
                    
                    currentStaff.addGamutSquare(currentGamutSquare, tempDurationNumber, tempTranspositionNumber);
                    
                    // remove just-selected cell from ArrayList.
                    unweightedArray.remove(currentRandom);
                }

			}
			else if (currentProcess==3)
			{
				// randomly select cells, use selection table with weightings
                //-----------------------------------------------------------
                
                // Make simple array of all cell numbers including multiple instances of each as dictated by weightings
                weightedArray = new ArrayList();
                
                tempCellcount = this.pieceGamut.getSize();                
                
                // Step through each GamutSquare, check their weighting value, and add n-instances
                // of that cell to the array. We will then later select indices from that array
                // randomly and use their corresponding values (cell numbers) as selected GamutSquares
                for(int k=0; k<tempCellcount;k++)
                {
                    tempWeight = this.pieceGamut.getGamutSquare(k).getWeighting();
                    
                    // loop "tempWeight" number of times to put multiple instances in array
                    for(int j=0; j<tempWeight; j++)
                    {
                        // Add one value for each weighting (so if weighting = 3 then we add that cell 3 times) 
                        weightedArray.add(new Integer(k));                        
                    }
                }
                
                // Use length of array - 1 (0 based) as uBound for random generator
                tempSize = weightedArray.size();
                
                // Same code as above but using tempSize
                while(currentStaff.getLength() < desiredlength)//this.length)
                {
                    currentRandom = generator.nextInt(tempSize);                     
                    tempValue = ((Integer)weightedArray.get(currentRandom)).intValue();
                                       
                     // Get random value's corresponding cell number from "weightedArray"                    
                    currentGamutSquare = (GamutSquare)this.pieceGamut.getGamutSquare(tempValue);
                    
                    currentStaff.addGamutSquare(currentGamutSquare, tempDurationNumber, tempTranspositionNumber);
                }
                                
			}
			else if (currentProcess==4)
			{
				// randomly select cells, use weightings table, remove cells from list when selected
                //-----------------------------------------------------------
                
                // Make simple array of all cell numbers including multiple instances of each as dictated by weightings
                weightedArray = new ArrayList();
                
                tempCellcount = this.pieceGamut.getSize();                
                
                // Step through each GamutSquare, check their weighting value, and add n-instances
                // of that cell to the array. We will then later select indices from that array
                // randomly and use their corresponding values (cell numbers) as selected GamutSquares
                for(int k=0; k<tempCellcount;k++)
                {
                    tempWeight = this.pieceGamut.getGamutSquare(k).getWeighting();
                    
                    // loop "tempWeight" number of times to put multiple instances in array
                    for(int j=0; j<tempWeight; j++)
                    {
                        // Add one value for each weighting (so if weighting = 3 then we add that cell 3 times) 
                        weightedArray.add(new Integer(k));                        
                    }
                }
                                
                // Same code as above but using tempSize
                while(currentStaff.getLength() < desiredlength && weightedArray.size() > 0) //< this.length && weightedArray.size() > 0)
                {
                    currentRandom = generator.nextInt(weightedArray.size());                     
                    tempValue = ((Integer)weightedArray.get(currentRandom)).intValue();
                                       
                     // Get random value's corresponding cell number from "weightedArray"                    
                    currentGamutSquare = (GamutSquare)this.pieceGamut.getGamutSquare(tempValue);
                    
                    currentStaff.addGamutSquare(currentGamutSquare, tempDurationNumber, tempTranspositionNumber);
                    
                    // remove just-selected cell from ArrayList. Do we have to shift the others??
                    weightedArray.remove(currentRandom);
                }                
			}				

			// Add currentStaff object to the Piece object's staffArray() ArrayList
			//this.staffArray.add(currentStaff);
			resultStaves.add(currentStaff);
		}
			
		return resultStaves;
	}
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Filename of the Gamut file.
  	**/
	public void setFile(String value)
	{
		this.File = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Filename of the Gamut file.
	**/
	public String getFile()
	{
		return this.File;
	}
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Name of the Piece.
  	**/
	public void setPieceName(String value)
	{
		this.piecename = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Name of the Piece
	**/
	public String getPieceName()
	{
		return this.piecename;
	}	
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Composer of the Piece.
  	**/
	public void setComposer(String value)
	{
		this.composer = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Composer of the Piece
	**/
	public String getComposer()
	{
		return this.composer;
	}		
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Poet of text for the Piece.
  	**/
	public void setPoet(String value)
	{
		this.poet = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Poet of text for the Piece
	**/
	public String getPoet()
	{
		return this.poet;
	}	

    /**
  	*	Public setter method
  	*
  	*	@param	value	double Length (double) in beats for the Piece.
  	**/
	public void setLength(double value)
	{
		this.length = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	double	Length (double) in beats for the Piece
	**/
	public double getLength()
	{
		return this.length;
	}
		
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Copyright data for the Piece.
  	**/
	public void setRights(String value)
	{
		this.rights = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Copyright data for the Piece
	**/
	public String getRights()
	{
		return this.rights;
	}
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Software-type data for the Piece.
  	**/
	public void setSoftware(String value)
	{
		this.software = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Software-type data for the Piece
	**/
	public String getSoftware()
	{
		return this.software;
	}		
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Encoding Date for the Piece.
  	**/
	public void setEncodingDate(String value)
	{
		this.encodingdate = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Encoding date for the Piece
	**/
	public String getEncodingDate()
	{
		return this.encodingdate;
	}		
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	int number of Systems for the Piece.
  	**/
	public void setSystems(int value)
	{
		this.systems = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	int	number of Systems for the Piece
	**/
	public int getSystems()
	{
		return this.systems;
	}		

    /**
  	*	Public setter method
  	*
  	*	@param	value	int number of Process for the Piece.
  	**/
	public void setProcess(int value)
	{
		this.process = value;
		this.pieceGamut.setProcess(value);
	}
	
	public void setProcess(String value)
	{		
		this.setProcess(new Integer(value).intValue());
	}
	
	/**
	*	Public getter method
	*
	*	@return	int	number of Process for the Piece
	**/
	public int getProcess()
	{
		return this.process;
	}		

    /**
  	*	Public setter method
  	*
  	*	@param	value	String file-name data for the Piece.
  	**/
	public void setOutputFilename(String value)
	{
		this.outputFilename = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	file-name data for the Piece
	**/
	public String getOutputFilename()
	{
		return this.outputFilename;
	}

    /**
  	*	Public setter method
  	*
  	*	@param	value	String Output file-path data for the Piece.
  	**/
	public void setOutputFilepath(String value)
	{
		this.outputFilepath = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Output file-path data for the Piece
	**/
	public String getOutputFilepath()
	{
		return this.outputFilepath;
	}

    /**
  	*	Public setter method
  	*
  	*	@param	value	String of output type data for the Piece.
  	**/
	public void setOutputType(String value)
	{
		this.outputType = value;
		
		if(this.pieceGamut!=null)
			this.pieceGamut.setOutputType(value.toUpperCase());
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	type of output data for the Piece
	**/
	public String getOutputType()
	{
		return this.outputType;
	}
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	Gamut Gamut object for the Piece.
  	**/
	public void setGamut(Gamut value)
	{
		this.pieceGamut = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	Gamut Gamut object for the Piece
	**/
	public Gamut getGamut()
	{
		return this.pieceGamut;
	}			
    
    private int getRandomValue(int gamutSize, Random ranObject)
    {
      //  Random generator = new Random(seed);                
        return ranObject.nextInt(gamutSize);    
    }
    
    public void writeDatafile(String filename)
    {
    		StringBuffer data = new StringBuffer();
    		
    		data = this.setHeaderText();
 /**   		
    		data = addLine(data, PIECENAME, this.pieceGamut.getPieceName());
    		data = addLine(data, COMPOSER, this.pieceGamut.getComposer());
    		data = addLine(data, POET, this.pieceGamut.getPoet());
    		data = addLine(data, RIGHTS, this.pieceGamut.getRights());
    		data = addLine(data, SOFTWARE, "jChing 1.0");//this.pieceGamut.getSoftware());
    		data = addLine(data, ENCODINGDATE, this.pieceGamut.getDate());
    		data = addLine(data, OUTPUTFILENAME, this.pieceGamut.getOutputFilename());
    		data = addLine(data, OUTPUTFILEPATH, this.pieceGamut.getOutputFilepath());
    		data = addLine(data, SYSTEMS, toString(this.pieceGamut.getSystems()));
    		data = addLine(data, SYSTEMCLEFS, this.pieceGamut.getSystemClefString());
    		data = addLine(data, PARTS, this.pieceGamut.getPartNamesString());
    		data = addLine(data, PROCESS, toString(this.pieceGamut.getProcess()));
    		data = addLine(data, SIZE, toString(this.pieceGamut.getSize()));
 **/
    		//printSystemOut(data.toString());
    		//printSystemOut("WERE READY TO WRITE OUT");
    		
    		// write StringBuffer "data" to .gam file
    		Write(data.toString(), filename);
    		
    	}
    
    public StringBuffer setHeaderText()
    {
		StringBuffer data = new StringBuffer();
			
		data = addLine(data, PIECENAME, this.pieceGamut.getPieceName());
		data = addLine(data, COMPOSER, this.pieceGamut.getComposer());
		data = addLine(data, POET, this.pieceGamut.getPoet());
		data = addLine(data, RIGHTS, this.pieceGamut.getRights());
		data = addLine(data, SOFTWARE, "jChing 1.0");//this.pieceGamut.getSoftware());
		data = addLine(data, ENCODINGDATE, this.pieceGamut.getDate());
		data = addLine(data, OUTPUTFILENAME, this.pieceGamut.getOutputFilename());
		data = addLine(data, OUTPUTFILEPATH, this.pieceGamut.getOutputFilepath());
		data = addLine(data, SYSTEMS, toString(this.pieceGamut.getSystems()));
		data = addLine(data, SYSTEMCLEFS, this.pieceGamut.getSystemClefString());
		data = addLine(data, PARTS, this.pieceGamut.getPartNamesString());
		data = addLine(data, PROCESS, toString(this.pieceGamut.getProcess()));
		data = addLine(data, SIZE, toString(this.pieceGamut.getSize()));
		
		return data;
    }
    
    private StringBuffer addLine(StringBuffer data, String tag, String value)
    {    		  		
    		if(value!=null)
    		{
    			data.append(tag + " " + value + nextline());
    		}
    		
    		return data;      
    }
    
    private StringBuffer addLine(StringBuffer data, String tag, Integer value)
    {
    		String temp = value.toString();
    		return addLine(data, tag, temp);
    }
    
    private StringBuffer addLine(StringBuffer data, String tag, int value)
    {
    		String temp = new Integer(value).toString();
    		return addLine(data, tag, temp);
    }
    
	public String getXmlOutput()
	{
		return xmlOutput;		
	}
	
	public void setXmlOutput(String value)
	{
		this.xmlOutput = value;
	}

	public void setXmlOutput(StringBuffer value)
	{
		this.xmlOutput = value.toString();
	}

	public String getLilyOutput()
	{
		return lilyOutput;		
	}
	
	public void setLilyOutput(String value)
	{
		this.lilyOutput = value;
	}

	public void setLilyOutput(StringBuffer value)
	{
		this.lilyOutput = value.toString();
	}
	
	public ArrayList getActivityLevels()
	{
		return this.pieceGamut.getActivityLevels();
	}
	
	public double getActivityMaximum()
	{
		return this.pieceGamut.getActivityMaximum();
	}
	
	public double getActivityMinimum()
	{
		return this.pieceGamut.getActivityMinimum();
	}
	
	public int getZones()
	{
		return this.zones;
	}
	
	public void setZones(int value)
	{
		this.zones = value;

		if(this.pieceGamut!=null)
			this.pieceGamut.setZones(value);		
	}	
	
	public double getBuildIncrement()
	{
		return this.buildincrement;
	}
	
	public void setBuildIncrement(double value)
	{
		this.buildincrement=value;
	}
}