import java.util.*;
import java.io.*;
import java.lang.reflect.*;

public class Xml extends Document
{	
	
	public String piecename;
	public String composer;
	public String poet;
	public String rights;
	public String software;
	public String encodingdate;
	public String[] instruments;
	public Piece currentPiece;

	public String lastDynamicValue = new String();
	public String xmlOutput;
	
	public int tabindex=0;
    public boolean firstMeasure=true;
    
    public boolean inChord = false;
    
    public int staffCounter=0;
    
    ArrayList processedChordNumbers = new ArrayList();
    
	/**
	*
	*	
	*	@param
	*	@value
	**/	
    public Xml()
    {
    
    }

    public Xml(boolean debug, Piece piece)
    {
       	this.setDebug(debug);       	
    	this.currentPiece = piece;    	
    	this.setPieceName(currentPiece.getPieceName());
    	this.setComposer(currentPiece.getComposer());
    	this.setPoet(currentPiece.getPoet());
    	this.setRights(currentPiece.getRights());
    	this.setSoftware(currentPiece.getSoftware());
    	this.setEncodingDate(currentPiece.getEncodingDate()); 	
    }
        
    /**
    *
    * Takes a passed-in List of all data and converts to XML
    *
    **/
	public void buildXML() //List line)
	{
		StringBuffer result = new StringBuffer();
		
		// Write the Header for the document
		// ---------------------------------
		result.append(this.writeHeader());
		
		// Start Partwise Score tags
		// -------------------------
		result.append("<score-partwise>");
        result.append(nextline());	
		result.append(this.tab(tabindex));				
		result.append(Wrap("movement-title", this.getPieceName()));			
		
		// Write Identification data
		// -------------------------
		result.append(this.tab(tabindex));
		result.append("<identification>");
		result.append(nextline());			
		result.append(this.tab(tabindex+1));
		result.append("<creator type=\"composer\">");
		result.append(this.getComposer());
		result.append("</creator>");
		result.append(nextline());	

		result.append(this.tab(tabindex));		
		result.append(Wrap("rights", this.getRights()));
		
		// Write Encoding data
		// -------------------------		
		result.append(this.tab(tabindex));		
		result.append("<encoding>");
		result.append(nextline());	
		result.append(this.tab(tabindex+1));		
		result.append(Wrap("encoding-date", this.getEncodingDate()));
//		result.append(Wrap("encoder", this.getEncoder);
		result.append(this.tab(tabindex));
		result.append(Wrap("software", this.getSoftware()));
//		result.append(Wrap("encoding-description", this.getEncodingDescription);
		result.append(this.tab(tabindex-1));
		result.append("</encoding>");
		result.append(nextline());
		
		// Write Source
		// ------------
//		result.append(Wrap("source", this.getSource);		
		
		// End Identification Data
		// -----------------------
		result.append(this.tab(tabindex-1));
		result.append("</identification>");
		result.append(nextline());	
		
		// Start Part-list tags
		// --------------------
		result.append(this.tab(tabindex));		
		result.append("<part-list>");
		result.append(nextline());
		result.append(this.tab(tabindex+1));				
		result.append(this.writePartHeaders());
		result.append(this.tab(tabindex-1));		
		result.append("</part-list>");
		
		// Write each part
		result.append(this.writeParts());
		
		// End tags for score-partwise/movement
		// ------------------------------------
		result.append("</score-partwise>");
		
		
		// write end comments which are the header data
		result.append(nextline());
		result.append("<!-- ");
		result.append(nextline());
		result.append(currentPiece.setHeaderText().toString());
		result.append(nextline());		
		result.append(" -->");
		setXmlOutput(result);
		currentPiece.setXmlOutput(result);
		
		// Print the result StringBuffer to file
        this.Write(result.toString(), this.currentPiece.getOutputFilepath() + this.currentPiece.getOutputFilename());
	}

	/**
	*
	*	
    *	For pieces with multiple staves, the first staff will be used as the template for measure lengths. All subsequent measures will 
    *	use the measure durations laid out by the first measure. This would be a good use for measurewise use of MusicXML.
    *
	*	@param
	*	@value
	**/		
	public StringBuffer writeNotes(int system) {
            
		StringBuffer result = new StringBuffer();		
	
        boolean method = true;         // TEMP METHOD
        
        if(method)
        {
            result = writeNoteDetails(system);
        }
        else
        {
            //result = writeNoteDetails2(system);            
        }
        					
		return result;	    
	}
    
    public StringBuffer writeNoteDetails(int system)
    {
        StringBuffer result = new StringBuffer();

		// Get current Staff object; set to 0 if process is 0 (just write out the cell data set)
        if(currentPiece.pieceGamut.getProcess()==0)
        		system = 0;
        
		Staff currentStaff = (Staff)this.currentPiece.staffArray.get(system);
        
        // Counter for multiple Staff measure-alignment
        this.staffCounter++;        
        
        boolean multiStaves = false;
        
        if(this.staffCounter>1 && currentPiece.pieceGamut.getProcess() > 0)
            multiStaves=true;
                
		// Get count of Gamut Squares from current Staff object
		int gamutSquareCount = currentStaff.gamutSquares.size();
		
        Staff masterStaff=new Staff(0); // first staff for use in multi-staved examples
        int masterGamutSquareCount = 0;        
        GamutSquare masterCurrentGamutSquare=new GamutSquare();
        int masterMeasureCount;
        
        // set first staff as guide for measure-length
        if(multiStaves) 
		{
            masterStaff = (Staff)this.currentPiece.staffArray.get(0);        
            masterGamutSquareCount = masterStaff.gamutSquares.size();                        
        }
        
        double thisCount;
        int measureCount=0;
        int previousMeasures=0;
        double currentMeasureLength=0;
        int totalMeasureCount=0;
                
		// Now step through each GamutSquare in the current Staff		
		for(int i=0; i<gamutSquareCount; i++)
		{
			GamutSquare currentGamutSquare = (GamutSquare)currentStaff.getGamutSquare(i);
			int noteCount = currentGamutSquare.Notes.size();            

            //This can cause an error
            //if(multiStaves)
                //masterCurrentGamutSquare = (GamutSquare)masterStaff.getGamutSquare(i);
                
            // ----------------------------------------------------------------------------
            // Start code for checking measure size
            // ----------------------------------------------------------------------------
            
            // Check how many measures are in the current gamut square
//            measureCount = currentGamutSquare.getMeasureCount()/2;
            // BUG HERE!!!!!! measure count is 2x what it should be, can't find out now, just cut it in 2
            measureCount = currentGamutSquare.getMeasureCount();                        
            printSystemOut("XML.java : writeNoteDetails() : currentGamutSquare.getMeasureCount: " + measureCount);
            printSystemOut("XML.java : writeNoteDetails() : currentGamutSquare.getActivityLevel: " + currentGamutSquare.getActivityLevel());            
            //if(multiStaves)
                //masterMeasureCount = masterCurrentGamutSquare.getMeasureCount();
                
            // track "last note read" for process 0 for multi-measure examples
            int nextNote=0;
            
            // Start a loop for each measure
            for(int measure=0;measure<measureCount;measure++)
            {                                         	            		
                // track global measure numbers
                totalMeasureCount = totalMeasureCount + measure;
                
                Measure currentMeasure = new Measure(totalMeasureCount);
                
                // Call method to determine lowest duration value for this measure (will step through all notes)
                double lowestSubdivision = findSubdivision(currentGamutSquare, measure);
                currentMeasure.setDivisions(lowestSubdivision);
                
                // Check duration (double value) of current measure 
                currentMeasureLength = currentGamutSquare.getMeasureLength(measure)/lowestSubdivision;                                                                                   

                currentMeasure.setDuration(currentMeasureLength);
                                    
                // THIS LINE WILL FUCK EVERYTHING UP SO BE CAREFUL!!!!!!
                //if(multiStaves)
                    //currentMeasureLength = masterCurrentGamutSquare.getMeasureLength(measure)/lowestSubdivision;
                    
                double currentMeasureSizeCounter=0;
                
                // start loop with nextNote to track multiple notes across multiple measures in one Gamut Square
                
                // temp string buffer for ordering of measure beat division
                StringBuffer tempResult = new StringBuffer();
                                          
                // keep track of lowest duration value to calculate durational division of quarternote
                double lowestDuration=1;

                int tempTabValue = this.tabindex;                
                    
                if(previousMeasures==0)
                {
                    // re-align tabs
                    this.setTab(tabindex+2);
                }
                else
                {
                    this.setTab(tabindex+1);
                }
                // Step through each note in the current Gamut Square				
                // Start loop for each note in system			
                for(int j=nextNote; j<noteCount; j++)
                {
					printSystemOut("Xml.java : writeNoteDetails() : notenumber=" + j);
					
                    thisCount=0;
                    double lengthCounter;                    
                    double curDuration=0;                    
                    
                    Note currentNote = currentGamutSquare.getNote(j);                                
					                    
                    if(j==noteCount-1)
                    	   currentNote.gsLastNote = true;
                    
                    thisCount = currentMeasureSizeCounter;
                    
                    // chordNoteNumber of 0 means no chord, value of 1 means first note in chord
                    if(currentNote.getChordNoteNumber()<2)
                    {
                        // adding note durations to MEASURE object
                        curDuration = currentNote.getDuration()/lowestSubdivision; 
                        
                        lengthCounter = currentMeasureSizeCounter+(curDuration);                        
                        currentMeasureSizeCounter=lengthCounter; 
                                
                    }
                    else // a second or later note in a chord, don't add any more duration to total
                    {
                        lengthCounter = currentMeasureSizeCounter;
                    }
                    
                    // if we're not over the length, write note
                    // if we are over, calculate smallest division of rest that fits and fill measure
                    // lengthCounter is the total duration plus the next note
                    // ------------------------------------------------------------------------------
                    if(lengthCounter<=currentMeasureLength)
                    {
                        
                     	// track lowest duration
                        if(curDuration<lowestDuration)
                        {
                            lowestDuration=curDuration;
                        }
                                                
                        // Add note duration to current Measure's ArrayList
                        currentMeasure.addNoteDuration(curDuration);  
                        
                        // Check random Dynamic setting on a Note-by-Note basis; don't set for process 0
                        if(currentPiece.pieceGamut.process!=0)
                        {
                        	tempResult.append(setDynamics(tabindex));
                        }
                        
                        tempResult.append(this.tab(tabindex));								
                        tempResult.append("<note>");
        
                        // Check if current note is a rest. If so, it has duration but not pitch(step or octave), or alter
                        if(!currentNote.getRest())
                        {                            
                            tempResult.append(nextline());
                            tempResult.append(this.tab(tabindex+1));
                            
                            if(currentNote.getChord())
                            {                            
                                if(currentNote.getChordNoteNumber()>1)
                                {
                                    tempResult.append("<chord/>");
                                    tempResult.append(nextline());
                                    tempResult.append(this.tab(tabindex));                                                                        
                                
                                    this.inChord = true;	// tell dynamics not to write more than once on a chord
                                }                       
                                else
                                {
                                	this.inChord = false;
                                }
                            }
                            else
                            {
                            	this.inChord=false;
                            }

                            tempResult.append("<pitch>");
                            
                            // Write Note value
                            tempResult.append(nextline());
                            tempResult.append(this.tab(tabindex+1));								
                            tempResult.append(this.Wrap(STEP, currentNote.getNoteValue()));
            
                            // Check for Accidental
                            if(currentNote.getAccidental().compareToIgnoreCase(NULL) != 0)
                            {
                                
                                // Don't write <alter> tag for naturals
                                if(currentNote.getAccidental()!="0")
                                {
                                    tempResult.append(this.tab(tabindex));                                
                                    tempResult.append(this.Wrap(ALTER, currentNote.getAccidental()));
                                }
                            }

                            // Write Octave value
                            tempResult.append(this.tab(tabindex));								
                            tempResult.append(this.Wrap(OCTAVE, this.makeString(currentNote.getOctaveNumber())));
                                                        
                            tempResult.append(this.tab(tabindex-1));												
                            tempResult.append("</pitch>");					
                        }
                        else
                        {
                            tempResult.append(nextline());
                            tempResult.append(this.tab(tabindex+1));					
                            tempResult.append("<rest/>");				
                        }
                
                        // DURATIONS ARE CALCULATED RELATIVE TO THE DIVISIONS VALUE, i.e. (divisions*duration)
                        // so if smallest division = 2, a quarter-note = duration of 2
                        
                        // Write Duration value
                        tempResult.append(nextline());
                        tempResult.append(this.tab(tabindex));												
                        tempResult.append(this.Wrap(DURATION, this.makeString(new Double(currentNote.getDuration()/lowestSubdivision).intValue())));
                        
                        // Write Duration string type representation
                        tempResult.append(this.tab(tabindex));												
                        tempResult.append(this.Wrap(TYPE, currentNote.getType()));

                        // if the duration has dotted values, 1 or more
                        int dots = currentNote.getDot();

                        for(int dotCount=0; dotCount<dots; dotCount++)
                        {
                            tempResult.append(this.tab(tabindex));
                            tempResult.append("<dot/>");	                            
                            tempResult.append(nextline());                           
                        }
                        
                        if(currentNote.getAccidentalType().compareToIgnoreCase(NULL) != 0)
                        {
                            // Write Accidental tag if exists
                            tempResult.append(this.tab(tabindex));
                            tempResult.append(this.Wrap(ACCIDENTALTYPE, currentNote.getAccidentalType()));
                        }
                        
                        int artCount = currentNote.articulations.size();
                        
                        if(artCount>0)
                        {
                        	tempResult.append(this.tab(tabindex));
                        	tempResult.append("<notations>");
                        	tempResult.append(nextline());                        	
                        	tempResult.append(this.tab(tabindex+1));
                        	tempResult.append("<articulations>");
                        	tempResult.append(nextline());
                        	tempResult.append(this.tab(tabindex+1));
                        	
                        	for(int a=0; a<artCount; a++)
                        	{
                        		String tempArt = currentNote.getArticulation(a);                        		
                        		tempResult.append("<" + tempArt + " placement=\"below\"/>"); 
                        		
                        		if(a<artCount-1)
                        		{
                        			tempResult.append(nextline());
                        			tempResult.append(this.tab(tabindex));
                        		}
                        	}
                        	
                        	tempResult.append(nextline());
                        	tempResult.append(this.tab(tabindex-1));
                        	tempResult.append("</articulations>");
                        	tempResult.append(nextline());
                        	tempResult.append(this.tab(tabindex-1));
                        	tempResult.append("</notations>");
                        	tempResult.append(nextline());
                        	
                        }
                        
                        tempResult.append(this.tab(tabindex-1));												
                        tempResult.append("</note>");
                        tempResult.append(nextline());                        
                        
                        // write heavy measure line for last note of gamut square on flat write-out
                        if(currentPiece.pieceGamut.getProcess()==0 && currentNote.gsLastNote)
                        {
                        		tempResult.append(this.tab(tabindex));
                        		tempResult.append("<barline location=\"right\">");
                        		tempResult.append(nextline());
                        		tempResult.append(this.tab(tabindex+1));
                        		tempResult.append("<bar-style>heavy</bar-style>");                            		
                        		tempResult.append(nextline());
                        		tempResult.append(this.tab(tabindex-1));
                        		tempResult.append("</barline>");
                        		tempResult.append(nextline());
                        		
                        		currentNote.gsLastNote = false;
                        }                        
                    }
                    // else if current note will make measure too long, replace with rest to fill measure                
                    else
                    {  
                        // check smallest subdivision used in this measure                      
                        double leftover = (currentMeasureLength-thisCount)*lowestSubdivision;
                        
                        if(leftover!=0)	// if we have a remainder...
                        {
                            // try to match a rest duration with leftover
                            
                            String noteType = new String();
                            
                            if(leftover==8)
                            {
                                noteType = "double-whole";
                            }
                            else if(leftover==4)
                            {
                                noteType = "whole";
                            }
                            else if(leftover==2)
                            {
                                noteType = "half";
                            }
                            else if(leftover==1)
                            {
                                noteType = "quarter";
                            }
                            else if(leftover==.5)
                            {
                                noteType = "eighth";
                            }
                            else if(leftover==.25)
                            {
                                noteType = "sixteenth";
                            }
                            else if(leftover==.125)
                            {
                                noteType = "thirty-second";
                            }
                            else
                            {
                                //printSystemOut("Xml.java: leftover duration " + leftover + " doesn't fit, need to add");                        
                            }                            
                            
                            // smallest subdivision that fits will be set to lowestDuration, to set measure.
                            // this way, remaining rests will fit the division scheme
                            
                            // fill measure with rests
                            tempResult.append(this.tab(tabindex));								
                            tempResult.append("<note>");
                            tempResult.append(nextline());
                            tempResult.append(this.tab(tabindex+1));					
                            tempResult.append("<rest/>");
                            tempResult.append(this.tab(tabindex));					
    
                            // Write Duration value
                            tempResult.append(nextline());
                            tempResult.append(this.tab(tabindex));												
                            tempResult.append(this.Wrap(DURATION, this.makeString(new Double(leftover/lowestSubdivision).intValue())));
                            
                            // Write Duration string type representation
                            tempResult.append(this.tab(tabindex));												
                            tempResult.append(this.Wrap(TYPE, noteType));                        
                            
                            // close Note
                            tempResult.append(this.tab(tabindex-1));												
                            tempResult.append("</note>");
                            tempResult.append(nextline());
                            
/*                            // write heavy measure line for last note of gamut square on flat write-out
                            if(currentPiece.pieceGamut.getProcess()==0 && currentNote.gsLastNote)
                            {
                            		tempResult.append(this.tab(tabindex));
                            		tempResult.append("<barline location=\"right\">");
                            		tempResult.append(nextline());
                            		tempResult.append(this.tab(tabindex+1));
                            		tempResult.append("<bar-style>heavy</bar-style>");                            		
                            		tempResult.append(nextline());
                            		tempResult.append(this.tab(tabindex-1));
                            		tempResult.append("</barline>");
                            		tempResult.append(nextline());
                            		
                            		currentNote.gsLastNote = false;
                            }
                            */
                            
                            // set value for currentMeasure note duration array
                            curDuration = leftover;                                                                              
                        }
                                                                                                                    
                        // set nextNote for next measure calculations
                        nextNote=j;
                        
                        // exit loop
                        j=noteCount;                                          
                    }    
                    
                    currentMeasure.addNoteDuration(curDuration);    
                    
                    //if(currentGamutSquare.getMeasureCount())
                }                
                
                // re-align tabs
                this.setTab(tempTabValue);
                                    
                // Start a new measure and append the measure tags to the result string
                result.append(writeMeasure(measure, currentGamutSquare, system, previousMeasures, lowestSubdivision));//lowestDuration));                                    
                
                // increment previous measure counter
                previousMeasures++;                    
                
                result.append(tempResult);  
       

                // Call method to write end closing measure tags (or just write here)

                // write end of measure tag
                result.append(this.tab(tabindex-1));						
                result.append("</measure>");
                result.append(nextline());
                this.firstMeasure=false;                

                // Add measure object to Staff object's array of measures
                currentStaff.addMeasure(currentMeasure);
            }            
		}
            
        return result;
    
    }
    
    public StringBuffer setDynamics(int curTabindex)
    {
    		StringBuffer tempResult = new StringBuffer();
    		
    		String value = currentPiece.pieceGamut.getDynamicValue();
    		
    		if(value!=NULL)
    		{
    			String curMIDIValue = currentPiece.pieceGamut.getDynamicMIDIValue(value); 
    			//String curDynamicValue = currentPiece.pieceGamut.getDynamicValue();
    			
    			// if the cur dynamic isn't the same as the last AND its not EMPTY and we're not on the nth note of a chord...
    			if(value.compareToIgnoreCase(this.lastDynamicValue)!=0 && value.compareToIgnoreCase(EMPTY)!=0 && !this.inChord)
    			{
    				tempResult.append(this.tab(tabindex));								
	    			tempResult.append("<direction placement=\"below\">");
	    			tempResult.append(nextline());
	    			tempResult.append(this.tab(tabindex+1));
	    			tempResult.append("<direction-type>");
	    			tempResult.append(nextline());
	    			tempResult.append(this.tab(tabindex+1));
	    			tempResult.append("<dynamics>");
	    			tempResult.append(nextline());
	    			tempResult.append(this.tab(tabindex+1));
	    			tempResult.append("<" + value + "/>");
	    			tempResult.append(nextline());
	    			tempResult.append(this.tab(tabindex-1));
	    			tempResult.append("</dynamics>");
	    			tempResult.append(nextline());
	    			tempResult.append(this.tab(tabindex-1));
	    			tempResult.append("</direction-type>");
	    			tempResult.append(nextline());	    		
	    		
	    			if(curMIDIValue!="-1")
	    			{	    				    		
	    				tempResult.append(this.tab(tabindex));
	    				tempResult.append("<sound dynamics =\"" + curMIDIValue +"\"/>");
	    				tempResult.append(nextline());
	    				
	    				// don't write lastValue for -1's like SFZ, they don't carry past the note
	    				this.lastDynamicValue = value;
	    			}
	    			
    				tempResult.append(this.tab(tabindex-1));
    				tempResult.append("</direction>");	
    				tempResult.append(nextline());	    				    			
    			}    			
    			
    		}
    		
        return tempResult;
    }
    
    
    
    public boolean multiStaves(int counter)
    {
        boolean returnValue = false;
        
        if(counter>1)
            returnValue = true;
        
        return returnValue;
    }
    
    public double findSubdivision(GamutSquare currentGamutSquare, int measure)
    {
        double thisCount=0;
        int noteCount = currentGamutSquare.Notes.size();        
        double currentMeasureLength = currentGamutSquare.getMeasureLength(measure);           
        double currentMeasureSizeCounter=0;
        double lowestDuration=1;        

        // track "last note read" for process 0 for multi-measure examples
        int nextNote=0;

        // Step through each note in the current Gamut Square				
        // Start loop for each note in system			
        for(int j=nextNote; j<noteCount; j++)
        {        
            Note currentNote = currentGamutSquare.getNote(j);
            double curDuration = currentNote.getDuration();
            double lengthCounter;        
            
            thisCount = currentMeasureSizeCounter;
            
            // chordNoteNumber of 0 means no chord, value of 1 means first note in chord
            if(currentNote.getChordNoteNumber()<2)
            {
                lengthCounter = currentMeasureSizeCounter+(curDuration);                               
                currentMeasureSizeCounter=lengthCounter;                        
            }
            else // a second or later note in a chord, don't add any more duration to total
            {
                lengthCounter = currentMeasureSizeCounter;
            }
            
            // if we're not over the length, write note
            // if we are over, calculate smallest division of rest that fits and fill measure
            // ------------------------------------------------------------------------------
            if(lengthCounter<=currentMeasureLength)
            {
                // track lowest duration
                if(curDuration<lowestDuration)
                {
                    lowestDuration=curDuration;
                }
                
            }
            // else if current note will make measure too long, replace with rest to fill measure                
            else
            {  
                // check smallest subdivision used in this measure                      
                double leftover = currentMeasureLength-thisCount;
                
                if(leftover>0 && leftover<lowestDuration)
                {
                    lowestDuration=leftover;
                }
                
                // set nextNote for next measure calculations
                nextNote=j;
                
                // exit loop
                j=noteCount;                
            }    
        }
        
        
        return lowestDuration;
    }
    
    
    public StringBuffer writeMeasure(int measureNumber, GamutSquare currentGamutSquare, int currentSystemNumber, int currentMeasureNumber, double lowestDuration)
    {
		StringBuffer result = new StringBuffer();		
        
        if(this.firstMeasure)
        {
            result.append(this.tab(tabindex+1));
        }
        else
        {
            result.append(this.tab(tabindex));
        }
        
//        int outputMeasureCount = measureNumber + 1;
        int outputMeasureCount = currentMeasureNumber + 1;
                
        result.append("<measure number=\"" + outputMeasureCount + "\">");	// measureNumber is 0-based
        
        result.append(nextline());	
        result.append(this.tab(tabindex+1));					
        result.append("<attributes>");
        result.append(nextline());			
        
        // division tags 
        result.append(this.tab(tabindex+1));			
        result.append("<divisions>");

        // need to keep track of the smallest note we use and based on that value, set the divisions value
        // for now make "8", means smallest division will be 8 divisions of a quarter note			
        //result.append("8");
        
        // This must correlate to the actual notes in the measure. We need to read all notes being output in the current measure and set the smallest subdivision of the quarter note here.
        Double divisor = new Double(1.0/lowestDuration);
        Integer intDivisor = new Integer(divisor.intValue());
        
        result.append(intDivisor.toString());
        
        result.append("</divisions>");
        result.append(nextline());					

        // ----------------------------------------------------------------------------
        // Key Signature and Time signature should come from Gamut Squares
        //  - if no value in GSquares, then take from piece.
        // ----------------------------------------------------------------------------
        // Key tags			
        result.append(this.tab(tabindex));						
        result.append("<key>");
        result.append(nextline());	
        result.append(this.tab(tabindex+1));	
        result.append("<fifths>");
        result.append("0");							// pull this from the Piece/measure object?
        result.append("</fifths>");			
        result.append(nextline());	
        result.append(this.tab(tabindex-1));				
        result.append("</key>");
        result.append(nextline());	
        
        // Time signature
        // --------------
        result.append(this.tab(tabindex));			
        result.append("<time>");
        result.append(nextline());				
        result.append(this.tab(tabindex+1));	

        result.append("<beats>");
        result.append(currentGamutSquare.getMeterNumerator(measureNumber));
//        result.append("4");							// pull this from the Piece object	
        result.append("</beats>");

        result.append(nextline());				
        result.append(this.tab(tabindex));	

        result.append("<beat-type>");        
        result.append(currentGamutSquare.getMeterDenominator(measureNumber));
//        result.append("4");							// pull this from the Piece object        
        result.append("</beat-type>");

        result.append(nextline());				
        result.append(this.tab(tabindex-1));	
        result.append("</time>");

        // clef tags 
        // ------------
        result.append(nextline());				
        result.append(this.tab(tabindex));			
        result.append("<clef>");
        result.append(nextline());			
        
        // clef sign = treble, bass, tenor, etc. as G plus line on which to act

        result.append(this.tab(tabindex+1));
        result.append("<sign>");
        result.append(getClefSign(currentSystemNumber));	// give get clef sign the current staff and return value
        result.append("</sign>");
        result.append(nextline());			

        result.append(this.tab(tabindex));
        result.append("<line>");
        result.append(getClefNumber(currentSystemNumber));	// give get clef line the current staff and return value
        result.append("</line>");		
        result.append(nextline());		
        result.append(this.tab(tabindex-1));				
        result.append("</clef>");														
        result.append(nextline());
        result.append(this.tab(tabindex-1));			
        result.append("</attributes>");	
        result.append(nextline());
        
        return result;
    }
    
    
	/**
	*
	*	
	*	@param
	*	@value
	**/	
	public StringBuffer writeHeader()
	{
		StringBuffer result = new StringBuffer();
		
		result.append("<?xml version=\"1.0\" standalone=\"no\"?>");
		result.append(nextline());
		result.append("<!DOCTYPE score-partwise PUBLIC \"-//Recordare//DTD MusicXML 0.7 Partwise//EN\" \"http://www.musicxml.org/dtds/partwise.dtd\">");
		result.append(nextline());

		return result;
	}
	
	
	// Get the number of systems or parts we want to generate and run writePart() that number of times
	public StringBuffer writePartHeaders()
	{
	
		// Go back later and pull part names from data file, not needed now		
		StringBuffer result = new StringBuffer();
		
		// Get number of systems/parts from the piece object
		int systemCount = this.currentPiece.getSystems();

		if(currentPiece.pieceGamut.getProcess() ==0)
			systemCount = 1;
		
		for(int i=0; i<systemCount; i++)
		{		
			result.append(writePartHeader(i+1));			
			
			if(i!=systemCount-1)
			{
				result.append(this.tab(tabindex));
			}
		}
		
		return result;
	}
	
	
	public StringBuffer writeParts()
	{
		StringBuffer result = new StringBuffer();
		
		// For each part (<part id ="P1">) etc.
		// ------------------------------------
//		int systemCount = this.currentPiece.getSystems();
		//int systemCount = this.currentPiece.staffArray.size();
		int systemCount = this.currentPiece.getSystems();
		
		//printSystemOut("number of StaffArraysize in Xml:writeParts: " +this.currentPiece.staffArray.size());
		//printSystemOut("number of getSystems from currentPiece: "+ this.currentPiece.getSystems());
		
		// Need to get actual number of systems, not just the number pulled from the .gam file
		
        result.append(nextline());		
        
        if(currentPiece.pieceGamut.getProcess() ==0)
        		systemCount=1;
        
		for(int i=0; i<systemCount; i++)
		{

			// write part id tags
			result.append(this.tab(tabindex));			
			result.append("<part id=\"P" + (i+1) + "\">");
			result.append(nextline());
            
            firstMeasure=true;
            
			// For each part write out notes with writeNotes(systemNumber)
            //printSystemOut("XML.java, writeParts-current system number is: "+i);
			result.append(this.writeNotes(i));

			// Write Parts Closer		
			// ------------------
            result.append(closeParts());
            
		}		
		
		return result;
	}

    public StringBuffer closeParts()
    {
		StringBuffer result = new StringBuffer();
        
//        result.append(this.tab(tabindex-1));						
//        result.append("</measure>");
//        result.append(nextline());
        
        result.append(this.tab(tabindex-1));						
        result.append("</part>");
        result.append(nextline());
        
        return result;
    }
	
	// Write Part will start building the score-partwise node and call from inside itself
	// each subsequent part. The wrapping for node start and end tags will occur in this hierarchical
	// manner
	public StringBuffer writePartHeader(int partNumber)
	{
		String part = new Integer(partNumber).toString();
		
		// Get parts from array generated at import using partNumber-1 as index
		
		StringBuffer result = new StringBuffer();
		
		// Get the parts from Piece object
		
		result.append("<score-part id = \"P" + part + "\">");
		result.append(nextline());		
		
		result.append(this.tab(tabindex+1));				
//		result.append("<part-name>quartet-" + part + "</part-name>");

		result.append("<part-name>" + (String)this.currentPiece.pieceGamut.getPartNames(partNumber-1) + "</part-name>");
		result.append(nextline());
		result.append(this.tab(tabindex-1));
		result.append("</score-part>");
		result.append(nextline());	
				
		return result;
	}
	

	
	/**
	*
	*	
	*	@param
	*	@value
	**/
	public String writeTag(String tagType, String value)
	{
		String tag;
		
		if(tagType.compareToIgnoreCase(NOTE) == 0)
		{	
			tag = Wrap(NOTE, value);
		}	
		else if(tagType.compareToIgnoreCase(ACCIDENTAL) == 0)
		{		
			tag = Wrap(ACCIDENTAL, value);
		}					
		else if(tagType.compareToIgnoreCase(REST) == 0)
		{		
			tag = Wrap(REST, value);
		}		
		else if(tagType.compareToIgnoreCase(CLEF) == 0)
		{		
			tag = Wrap(CLEF, value);			
		}
		else
		{			
			tag = BLANK;		// need to catch this occurance if it happens
		}					
				
		return tag;
	}
	

	/**
	*
	*	
	*	@param
	*	@value
	**/	
	public StringBuffer buildKeySignature(String value)
	{
		StringBuffer result = new StringBuffer();
		
		result.append(this.Wrap(KEY, this.Wrap(FIFTHS, value)));

		return result;		
	}
	

	/**
	*
	*	
	*	@param
	*	@value
	**/	
	public String Wrap(String outerNode, String value)
	{
		StringBuffer result = new StringBuffer();
		
		result.append("<"+outerNode+">");
		result.append(value);
		result.append("</"+outerNode+">");
		result.append(nextline());
		
		return result.toString();
	}

	/**
	*
	*	
	*	@param
	*	@value
	**/	
	public String Wrap(String outerNode, double value)
	{
		StringBuffer result = new StringBuffer();
		
		result.append("<"+outerNode+">");
		result.append(value);
		result.append("</"+outerNode+">");
		result.append(nextline());
		
		return result.toString();
	}
		
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Name of the Piece.
  	**/
	public void setPieceName(String value)
	{
		this.piecename = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Name of the Piece
	**/
	public String getPieceName()
	{
		return this.piecename;
	}	
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Composer of the Piece.
  	**/
	public void setComposer(String value)
	{
		this.composer = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Composer of the Piece
	**/
	public String getComposer()
	{
		return this.composer;
	}		
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Poet of text for the Piece.
  	**/
	public void setPoet(String value)
	{
		this.poet = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Poet of text for the Piece
	**/
	public String getPoet()
	{
		return this.poet;
	}	
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Copyright data for the Piece.
  	**/
	public void setRights(String value)
	{
		this.rights = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Copyright data for the Piece
	**/
	public String getRights()
	{
		return this.rights;
	}
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Software-type data for the Piece.
  	**/
	public void setSoftware(String value)
	{
		this.software = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Software-type data for the Piece
	**/
	public String getSoftware()
	{
		return this.software;
	}		
	
    /**
  	*	Public setter method
  	*
  	*	@param	value	String Encoding Date for the Piece.
  	**/
	public void setEncodingDate(String value)
	{
		this.encodingdate = value;
	}
	
	/**
	*	Public getter method
	*
	*	@return	String	Encoding date for the Piece
	**/
	public String getEncodingDate()
	{
		return this.encodingdate;
	}			

	public String getClefSign(Integer value)
	{
		// For the current staff, get the appropriate clef sign
		// these are marked in data file as "CLEFS T/T/A/G" etc.
		// stored as gamut.getSystemClefs() which is an arraylist
		// get this.currentPiece.pieceGamut.getSystemClefs(value)
		return this.currentPiece.pieceGamut.getSystemClefs(value);
	}
		
	public String getClefSign(int value)
	{
		// For the current staff, get the appropriate clef sign
		// these are marked in data file as "CLEFS T/T/A/B" etc.
		// stored as gamut.getSystemClefs() which is an arraylist
		// get this.currentPiece.pieceGamut.getSystemClefs(value)
		
		return this.currentPiece.pieceGamut.getSystemClefs(value);
	}
	
	public String getClefNumber(int value)
	{
		return this.currentPiece.pieceGamut.getSystemClefNumbers(value);
	}

	public String getClefNumber(Integer value)
	{
		return this.getClefNumber(value);
	}

	public String tab(int value)
	{
		StringBuffer result = new StringBuffer();
		
		for(int i=0; i<= value; i++)
		{
			result = result.append(TAB);	
		}	

		this.tabindex = value;
				
		return result.toString();
	}
	
    public void setTab(int value)
    {
        this.tabindex=value;
    }
    
	public String makeString(Integer value)
	{
		return value.toString();
	}
	
	public String makeString(int value)
	{
		Integer tempInteger = new Integer(value);

		return tempInteger.toString();		
	}

	public String makeString(double value)
	{
		Double tempDouble = new Double(value);

		return tempDouble.toString();		
	}
	
	public String getXmlOutput()
	{
		return xmlOutput;		
	}
	
	public void setXmlOutput(String value)
	{
		this.xmlOutput = value;
	}

	public void setXmlOutput(StringBuffer value)
	{
		this.xmlOutput = value.toString();
	}
}